<?php
if(!defined('ABSPATH')) exit;

class EventM_Do_Actions {
    public function __construct(){
        add_action('event_magic_booking_confirmed',array($this,'booking_confirmed'),10,1);
        add_action('event_magic_booking_cancelled',array($this,'booking_cancelled'),10,1);
        add_action('wp_head',array($this,'twitter_meta'));
        add_filter('tribe_events_rewrite_rules_custom',array($this,'tribe_events_rewrite_rules_custom'),10,3);
        add_filter('event_magic_refund_booking',array($this,'refund_booking'));
        add_filter('em_cpt_event',array($this,'change_post_type_info'));
    }
    
    public function after_seat_booked($booking){
        $event_service = EventM_Factory::get_service('EventM_Service');
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        $event= $event_service->load_from_db($booking->event);
        if(empty($event->ticket_price)){ // Handling 0 price booking
            // Changing booking status
            $booking->status='completed';
            $booking = $booking_service->save($booking);
            EventM_Notification_Service::booking_confirmed($booking->id);
        }
        else{
            if($booking->status=='pending'){
                $booking->status='completed';
                $booking = $booking_service->save($booking);
                EventM_Notification_Service::booking_confirmed($booking->id);
            }
        }
    }
    
    public function booking_cancelled($booking){
         EventM_Notification_Service::booking_cancel($booking);
    }
    
    public function booking_confirmed($booking){
        EventM_Notification_Service::booking_confirmed($booking);
    }
    
    public function twitter_meta(){
        global $wp;
        $event_id= isset($_GET['event']) ? absint($_GET['event']) : false;
        if(empty($event_id))
            return;
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $options= $setting_service->load_model_from_db();
       
        if(empty($options->social_sharing))
            return;
        
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($event_id);
        if(!empty($event->cover_image_id)){
            $image = wp_get_attachment_image_url($event->cover_image_id, 'full');
        }
        else{
             $image = esc_url(EM_BASE_URL.'includes/templates/images/dummy_image.png');
        }
        echo '<meta name="twitter:card" content="summary_large_image"/> ';
        echo "<meta name='twitter:title' content='".strip_tags($event->name)."'>";
        echo "<meta name='twitter:description' content='".strip_tags($event->description)."'>";
        echo "<meta name='twitter:image' content='$image'>";
        
        $current_url =home_url(add_query_arg(array('event'=>$event->id),$wp->request));
        echo '<meta property="og:url" content="'.$current_url.'" />';
        echo '<meta property="og:title" content="'.strip_tags($event->name).'" />';
        echo '<meta property="og:description" content="'.strip_tags($event->description).'"/>';
        echo '<meta property="og:image" content="'.$image.'" />';
        echo '<meta property="og:type" content="article"/>';
        echo '<meta property="fb:app_id" content="'.$options->fb_api_key.'">';
    }
    
    /*
     * Removes rewrite rule for "/events" page. 
     */
    public function tribe_events_rewrite_rules_custom($rules, $obj, $wp_rewrite){
        $found = false;
        foreach ($rules as $rule => $rewrite ) {
            if (preg_match( '/\?:events\)/', $rule ) ) {
              unset( $rules[$rule] );
              $found= true;
            }
        }
        if($found){
            add_filter('show_admin_bar', '__return_false');
        }
        return $rules;
    }
    
    public function refund_booking($booking){
        if(isset($booking->order_info['payment_gateway'])){
            if ($booking->order_info['payment_gateway'] == "paypal") {
                $payment_service = EventM_Factory::get_service('EventM_Paypal_Service');
                $payment_response = $payment_service->refund($booking);
                if (!empty($payment_response) && $payment_response['ACK'] == "Success") {
                    $booking->status = 'refunded';
                }
            }
        }
        return $booking;
    }
    
    /*
     * Resetting publicly queryable for 'events/?event=' frontend URLs. 
     */
    
    function change_post_type_info($info) {
        if (!empty($info['publicly_queryable'])) {
            $info['publicly_queryable']= false;
        }
        return $info;
    }
}

new EventM_Do_Actions();