<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class EventM_AJAX {

    protected $request;
    protected $services= array();
    private static $instance;
    
    public static function get_instance()
    {   
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }
    
    private function __construct() {
        $this->services['venue']= EventM_Factory::get_service('EventM_Venue_Service');
        $this->services['event']= EventM_Factory::get_service('EventM_Service');
        $this->services['performer']= EventM_Factory::get_service('EventM_Performer_Service');
        $this->services['event_type']= EventM_Factory::get_service('EventTypeM_Service');
        $this->services['setting']= EventM_Factory::get_service('EventM_Setting_Service');
        $this->services['booking']= EventM_Factory::get_service('EventM_Booking_Service');
        $this->request = EventM_Raw_Request::get_instance();
        $this->add_ajax_events();
    }

    public function add_ajax_events() {
        add_action('wp_ajax_em_load_strings', array($this, 'load_data'));
        $ajax_events = array('save_venue' => false,
            'save_performer' => false,
            'save_event_type' => false,
            'save_event' => false,
            'save_popup_event'=>false,
            'save_dropped_event'=>false,
            'delete_posts' => false,
            'duplicate_posts' => false,
            'delete_terms' => false,
            'duplicate_terms' => false,
            'save_global_settings' => false,
            'save_booking' => false,
            'cancel_booking' => false,
            'load_event_dates' => true,
            'load_venue_addresses' => true,
            'register_user' => true,
            'check_bookable' => true,
            'login_user' => true,
            'book_seat' => true,
            'load_event_seats' => true,
            'load_payment_configuration' => true,
            'update_booking' => true,
            'print_ticket' => false,
            'export_bookings' => false,
            'event_details' => true,
            'show_booking_details' => false,
            'cancel_booking_by_user' => false,
            'get_venue_capcity' => true,
            'verify_booking' => true,
            'download_booking_details' => true,
            'confirm_booking_without_payment' => true,
            'rm_custom_datas' => true,
            'resend_mail' => true,
            'booking_cancellation_mail' => true,
            'booking_confirm_mail' => true,
            'booking_refund_mail' => true,
            'booking_pending_mail' => true,
            'load_event_for_booking' => true,
            'delete_order' => true,
            'add_event_tour_completed' => false,
            'event_tour_completed' => false,
            'load_admin_events'=> false,
            'deactivation_feedback'=> false,
            'frontend_event_submit_form_strings' => true,
            'submit_frontend_event' => true,
            'event_submitted_mail' => true,
            'event_approved_mail' => false,
            'upload_image_from_frontend' => true
        );
        foreach ($ajax_events as $ajax_event => $nopriv) {
            add_action('wp_ajax_em_' . $ajax_event, array($this, $ajax_event));
            if ($nopriv) {
                add_action('wp_ajax_nopriv_em_' . $ajax_event, array($this, $ajax_event));
            }
        }
    }

    public function load_data() {
        $context = event_m_get_param('em_request_context');
        if(empty($context))
            return;
        if(method_exists($this,$context)){
            $this->{$context}();
        }
        do_action('event_magic_load_strings', $context);
    }
    
    // Loads events for admin screen.
    public function load_admin_events(){
        $this->check_permission();
        $view= event_m_get_param('view');
        $response= $view!='cards' ? $this->services['event']->admin_calendar_view() : $this->services['event']->load_list_page();
        wp_send_json_success($response);
    }
    
    private function admin_venues(){
        $this->check_permission();
        $response= $this->services['venue']->load_list_page();
        wp_send_json_success($response);
    }
    private function admin_venue(){
        $this->check_permission();
        $venue= $this->services['venue']->load_edit_page();
        wp_send_json_success(array('term'=>$venue));
    }
    
    private function admin_event(){
        $this->check_permission();
        $response= $this->services['event']->load_edit_page();
        wp_send_json_success($response);
    }
    
    private function admin_performer(){
        $this->check_permission();
        $performer= $this->services['performer']->load_edit_page();
        wp_send_json_success(array('post'=>$performer));
    }
    
    private function admin_performers(){
        $this->check_permission();
        $response= $this->services['performer']->load_list_page();
        wp_send_json_success($response);
    }
    
    private function admin_event_type(){
        $this->check_permission();
        $response= $this->services['event_type']->load_edit_page();
        wp_send_json_success(array('term'=>$response));
    }
    
    private function admin_event_types(){
        $this->check_permission();
        $response=$this->services['event_type']->load_list_page();
        wp_send_json_success($response);
    }
    
    private function admin_global_settings(){
        $this->check_permission();
        $options= $this->services['setting']->load_edit_page();
        wp_send_json_success(array('options'=>$options));
    }
    
    private function admin_bookings(){
        $this->check_permission();
        $bookings= $this->services['booking']->load_bookings();
        wp_send_json_success($bookings);
    }
    
    private function admin_booking(){
        $this->check_permission();
        $booking= $this->services['booking']->load_booking();
        wp_send_json_success(array('post'=>$booking));
    }
    
    private function admin_event_settings(){
        $this->check_permission();
        $event_settings= $this->services['event']->load_settings_page();
        wp_send_json_success(array('post'=>$event_settings));
    }
    
    public function save_venue() {
        $this->check_permission();
        $model = $this->request->map_request_to_model('EventM_Venue_Model');
        $model->id = absint(event_m_get_param('id'));

        // Validate data
        $errors = $this->services['venue']->validate($model);
        if (!empty($errors)) {
            wp_send_json_error(array('errors'=>$errors));
        }
        if ($model->type == "standings") {
            $model->seats = array();
            $model->seating_capacity = 0;
        }
        if(!empty($model->established)){
            $model->established= em_time($model->established);
        }
        $venue = $this->services['venue']->save($model);
        if ($venue instanceof WP_Error) {
            $error_msg= $venue->get_error_message(); 
            wp_send_json_error(array('errors'=>array($error_msg)));
        }
        else{
            //If type is seating then update capacity in the corresponding event if it is greater than venue capacity
            $events = $this->services['event']->get_events_by_venue($venue->id);
            foreach($events as $event){
                if(empty($event->seats)){
                    if($venue->type=='seats'){
                        $event->seating_capacity= $venue->seating_capacity;
                        $event->seats=$venue->seats;
                    }
                    else{
                        $event->seats=array();
                    }
                    $this->services['event']->update_model($event);
                }
            }
        }
        $redirect = admin_url('admin.php/?page=em_venues');
        wp_send_json_success(array('redirect'=>$redirect));
    }

    public function save_performer() {
        $this->check_permission();
        $model = $this->request->map_request_to_model('EventM_Performer_Model');
        $model->id = absint(event_m_get_param('id'));
        $performer = $this->services['performer']->save($model);
        
        if ($performer instanceof WP_Error) {
            $error_msg= $performer->get_error_message(); 
            wp_send_json_error(array('errors'=>array($error_msg)));
        }
        $redirect = admin_url('/admin.php?page=em_performers');
        wp_send_json_success(array('redirect'=>$redirect));
    }

    public function save_event_type(){
        $this->check_permission();
        $model = $this->request->map_request_to_model('EventM_Event_Type_Model');
        $model->id = absint(event_m_get_param('id'));

        // Validate data
        $errors = $this->services['event_type']->validate($model);
        if (!empty($errors)) {
            wp_send_json_error(array('errors'=>$errors));
        }

        $event_type = $this->services['event_type']->save($model);
        if ($event_type instanceof WP_Error) {
            $error_msg= $event_type->get_error_message(); 
            wp_send_json_error(array('errors'=>array($error_msg)));
        }
        $redirect = admin_url('admin.php/?page=em_event_types');
        wp_send_json_success(array('redirect'=>$redirect));
    }

    public function save_event() {
        $this->check_permission();
        $id= absint(event_m_get_param('id'));
        if(empty($id)){
            wp_send_json_error(array('errors'=>array(__('Invalid event ID','eventprime-event-calendar-management'))));
        }
        $screen= event_m_get_param('screen');
        // Loading existing model
        $event= $this->services['event']->load_model_from_db($id);
        if(empty($event->id)){
            wp_send_json_error(array('errors'=>array(__('Invalid event ID','eventprime-event-calendar-management'))));
        }
        
        if($screen=='event_settings'){ // Saving Event Setting screen options
            $errors=array();
            $old_status= $event->status;
            // Event type validation
            $event_type = sanitize_text_field(event_m_get_param('event_type'));
            if ($event_type=='new_event_type')
            {
                $new_event_type= event_m_get_param('new_event_type');
                if(term_exists($new_event_type,EM_EVENT_TYPE_TAX)){
                    $errors[]= __('Please use different Event Type','eventprime-event-calendar-management');
                }
            }
            
            // Dates validation
            $start_date= em_timestamp(event_m_get_param('start_date'));
            $end_date= em_timestamp(event_m_get_param('end_date'));
            if ($start_date > $end_date) {
                $errors[]= __('Event Start date should be prior to end date.','eventprime-event-calendar-management');
            }
            $enable_booking= absint(event_m_get_param('enable_booking'));
            if(!empty($enable_booking)){
                $start_booking_date =  em_timestamp(event_m_get_param('start_booking_date'));
                $last_booking_date = em_timestamp(event_m_get_param('last_booking_date'));
                if ($last_booking_date> $end_date) {
                    $errors[]=__('Last booking date can not be greater than End date.','eventprime-event-calendar-management');
                }

                if ($start_booking_date > $last_booking_date) {
                    $errors[]=__('Start booking date should be earlier than the Last Booking date.','eventprime-event-calendar-management');
                }
                if ($start_booking_date > $start_date) {
                    $errors[]=__('Start booking date must be earlier than Event Start date.','eventprime-event-calendar-management');
                }
                if ($start_booking_date == $last_booking_date) {
                    $errors[]=__('Start booking date must be earlier than end booking date.','eventprime-event-calendar-management');
                }
            }
            if(absint(event_m_get_param('custom_link_enabled')) == 1){
                if(!preg_match('/^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/',event_m_get_param('custom_link'))){
                    $errors[]=__('Incorrect custom link. Please update custom link to a correct URL format.','eventprime-event-calendar-management');
                }
            }
            $errors= apply_filters('em_event_settings_validation',$errors);
            if(!empty($errors)){
                wp_send_json_error(array('errors'=>$errors));
            }
            $this->services['event']->save_settings($event);
        }
        else if($screen=='event_performers'){
            $this->services['event']->save_performers($event);
        }
        else if($screen=='event_organizer'){
            $errors = array();
            $emails = event_m_get_param('organizer_emails');
            $websites = event_m_get_param('organizer_websites');
            
            foreach($emails as $email) {
                if (!preg_match('/^([a-zA-Z0-9_\-\.]+)@([a-zA-Z0-9_\-\.]+)\.([a-zA-Z]{2,5})$/',$email)) {
                    $errors[]=__('Incorrect email format.','eventprime-event-calendar-management');
                    break;
                }
            }
            
            foreach($websites as $website) {
                if (!preg_match('/^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/',$website)) {
                    $errors[]=__('Incorrect website URL format.','eventprime-event-calendar-management');
                    break;
                }
            }
            
            if (!empty($errors)) {
                wp_send_json_error(array('errors'=>$errors));
            }

            $this->services['event']->save_organizer($event);
        }
        else if($screen=='event_social'){
            $this->services['event']->save_social($event);
        }
        else if($screen=='event_venue'){
            $errors= apply_filters('em_event_capacity_validation',$errors,$event);
            if(!empty($errors)){
                wp_send_json_error(array('errors'=>$errors));
            }
            $this->services['event']->save_venue($event);
        }
        
        do_action('event_magic_save_event',$event->id);
        $redirect = admin_url('/admin.php?page=em_dashboard&post_id=' . $event->id);
        $response['redirect'] = $redirect;
        // event data for email
        if($screen=='event_settings'){
            $event= $this->services['event']->load_model_from_db($event->id);
            $new_status= $event->status;
            $cal_event= array();
            $cal_event['id']= $event->id;
            $cal_event['title']= $event->name;
            $cal_event['start']= date('c',$event->start_date);
            $cal_event['start_booking_date']= $event->start_booking_date;
            $cal_event['end']= date('c',$event->end_date);
            $cal_event['last_booking_date']= $event->last_booking_date;
            $cal_event['enable_booking']= $event->enable_booking;
            $cal_event['ticket_price']= $event->ticket_price;
            $cal_event['all_day']= $event->all_day;
            $cal_event['performer']= $event->performer;
            $cal_event['venue']= $event->venue;
            $cal_event['event_type']= $event->event_type;
            $cal_event['status']= $event->status;
            $cal_event= apply_filters('ep_admin_calendar_event',$cal_event,$event);
            $response['event'] = $cal_event;
            if($old_status == 'draft' && $new_status == 'publish' && $event->user_submitted == 1) {
                $response['status_changed'] = true;
            } else {
                $response['status_changed'] = false;
            }
        }
        wp_send_json_success($response);
    }

    public function save_dropped_event(){
        $this->check_permission();
        $id = absint(event_m_get_param('id'));
        $event= $this->services['event']->load_model_from_db($id);
        $event->start_date= em_timestamp(event_m_get_param('start_date'));
        $event->end_date= em_timestamp(event_m_get_param('end_date'));
        $event->start_booking_date= em_get_local_timestamp();
        $event->last_booking_date= $event->start_date;
        $event_id = $this->services['event']->save_popup_event($event);
        $event= $this->services['event']->load_model_from_db($event_id);
        $cal_event= array();
        $cal_event['id']= $event->id;
        $cal_event['title']= $event->name;
        $cal_event['start']= date('c',$event->start_date);
        $cal_event['start_booking_date']= $event->start_booking_date;
        $cal_event['end']= date('c',$event->end_date);
        $cal_event['last_booking_date']= $event->last_booking_date;
        $cal_event['enable_booking']= $event->enable_booking;
        $cal_event['ticket_price']= $event->ticket_price;
        $cal_event['allDay']= $event->all_day;
        $cal_event['performer']= $event->performer;
        $cal_event['venue']= $event->venue;
        $cal_event['popup']= $this->services['event']->admin_event_hover_popup($event);
        $redirect = admin_url('admin.php?page=em_dashboard&post_id='.$event->id);
        $em = event_magic_instance(); 
        if(in_array('recurring_events',$em->extensions) && !empty(em_get_child_events($event->id))) {
            wp_send_json_success(array('event'=>$cal_event,'redirect'=>$redirect,'reload'=>true));
        } else {
            wp_send_json_success(array('event'=>$cal_event,'redirect'=>$redirect,'reload'=>false));
        }
    }
    
    public function save_popup_event(){
        $this->check_permission();
        $id = absint(event_m_get_param('id'));
        $event= $this->services['event']->load_model_from_db($id);
        $old_status= $event->status;
        $errors= array();
        // Data validation
        $name= htmlspecialchars_decode(event_m_get_param('title'));
        if(empty($name)){
            array_push($errors,__('Event name can not be empty.','eventprime-event-calendar-management'));
        }
        $start_date= em_timestamp(event_m_get_param('start_date'));
        $end_date= em_timestamp(event_m_get_param('end_date'));
        if(empty($start_date) || empty($end_date)){
            array_push($errors,__('Invalid event dates.','eventprime-event-calendar-management'));
        }
        if($end_date<=$start_date){
            array_push($errors,__('Start date should be prior to End date.','eventprime-event-calendar-management'));
        }
        if (!empty($errors)) {
            wp_send_json_error(array('errors'=>$errors));
        }
        
        $event_id = $this->services['event']->save_popup_event($event);
        $event= $this->services['event']->load_model_from_db($event_id);
        $new_status= $event->status;
        $cal_event= array();
        $cal_event['id']= $event->id;
        $cal_event['title']= $event->name;
        $cal_event['start']= date('c',$event->start_date);
        $cal_event['start_booking_date']= $event->start_booking_date;
        $cal_event['end']= date('c',$event->end_date);
        $cal_event['last_booking_date']= $event->last_booking_date;
        $cal_event['enable_booking']= $event->enable_booking;
        $cal_event['ticket_price']= $event->ticket_price;
        $cal_event['all_day']= $event->all_day;
        $cal_event['performer']= $event->performer;
        $cal_event['venue']= $event->venue;
        $cal_event['event_type']= $event->event_type;
        $cal_event['status']= $event->status;
        $cal_event['popup']= $this->services['event']->admin_event_hover_popup($event);
        $cal_event= apply_filters('ep_admin_calendar_event',$cal_event,$event);
        $redirect = admin_url('admin.php?page=em_dashboard&post_id='.$event->id);
        $em = event_magic_instance();
        
        $response = array(
            'event'=>$cal_event,
            'redirect'=>$redirect
        );
        
        if($old_status == 'draft' && $new_status == 'publish' && $event->user_submitted == 1) {
            $response['status_changed'] = true;
        } else {
            $response['status_changed'] = false;
        }
        
        if(in_array('recurring_events',$em->extensions) && !empty(em_get_child_events($event->id))) {
            $response['reload'] = true;
        } else {
            $response['reload'] = false;
        }
        
        wp_send_json_success($response);
    }
    
    function delete_posts() {
        $this->check_permission();
        $ids= $this->request->get_param('ids');
        if(!empty($ids)){
            if(!is_array($ids)){
                $ids = explode(',', $ids);
            }
            foreach ($ids as $id) {
                $post = get_post($id);
                if($post->post_type == 'em_booking' && $post->post_status == 'completed'){
                    $response = new stdClass();
                    $response->error = true;
                    $response->message = __("Booking with Completed status will not be  delete directly. First Cancel the Booking and then try again", 'eventprime-event-calendar-management');
                    echo json_encode($response);
                    wp_die();
                }
                $child_posts = get_posts(
                    array(
                        'post_parent' => $id,
                        'post_type' => EM_EVENT_POST_TYPE,
                        'status' => 'any'
                    )
                );
                if(!empty($child_posts)) {
                    foreach ($child_posts as $child_post) {
                        wp_delete_object_term_relationships($child_post->ID, array(EM_EVENT_VENUE_TAX, EM_EVENT_TYPE_TAX));
                        wp_delete_post($child_post->ID);
                    }
                }
                wp_delete_object_term_relationships($id, array(EM_EVENT_VENUE_TAX, EM_EVENT_TYPE_TAX));
                //wp_delete_post($id,true);
                $post = array(
                    'ID' => $id,
                    'post_status' => 'trash',
                );
                wp_update_post($post);
            }
        }
        wp_die();
    }

    function delete_terms() {
        $this->check_permission();
        $tax_type= $this->request->get_param('tax_type');
        $ids=$this->request->get_param('ids');
        if(!empty($ids)){
            if(!is_array($ids)){
                $ids = explode(',', $ids);
            }
            foreach ($ids as $id) {
                $posts = em_get_attached_posts($id, $tax_type);
                foreach ($posts as $post):
                    $p = array(
                        'ID' => $post->ID,
                        'post_status' => 'trash'
                    );
                    wp_update_post($p);
                endforeach;
                wp_delete_term($id, $tax_type);
            }
        }
        $response = new stdClass();
        $response->reload = true;
        echo json_encode($response);
        wp_die();
    }

    function duplicate_posts() {
        $this->check_permission();
        $ids=$this->request->get_param('ids');
        if(!empty($ids)){
            if(!is_array($ids)){
                $ids = explode(',', $ids);
            } 
            $new_post_id = 0;
            foreach ($ids as $key =>  $id) {
                $post = get_post($id);
                $current_user = wp_get_current_user();
                $new_post_author = $current_user->ID;
                if (isset($post) && $post != null) {
                    $args = array(
                        'comment_status' => $post->comment_status,
                        'ping_status' => $post->ping_status,
                        'post_author' => $new_post_author,
                        'post_content' => $post->post_content,
                        'post_excerpt' => $post->post_excerpt,
                        'post_name' => $post->post_name,
                        'post_parent' => $post->post_parent,
                        'post_status' => $post->post_status,
                        'post_title' => $post->post_title,
                        'post_type' => $post->post_type,
                        'menu_order' => $post->menu_order
                    );
                    $new_post_id = wp_insert_post($args);
                    // Copying Event Type taxonomy
                    $event_type_term = wp_get_object_terms($id, EM_EVENT_TYPE_TAX, array('fields' => 'slugs'));
                    if (!empty($event_type_term))
                        wp_set_object_terms($new_post_id, $event_type_term, EM_EVENT_TYPE_TAX, false);

                    // Copying Venue taxonomy
                    $venue_term = wp_get_object_terms($id, EM_EVENT_VENUE_TAX, array('fields' => 'slugs'));
                    if (!empty($venue_term))
                        wp_set_object_terms($new_post_id, $venue_term, EM_EVENT_VENUE_TAX, false);

                    /*
                     * duplicate all post meta just
                     */
                    $data = get_post_custom($id);
                    foreach ($data as $key => $values) {
                        if ($key == em_append_meta_key('seats')) {
                            $event_service = EventM_Factory::get_service('EventM_Service');
                            $event_service->create_seats_from_venue($new_post_id, $event_service->get_venue($id));
                            continue;
                        }
                        if (in_array($key, em_append_meta_key(array('booked_seats')))) {
                            continue;
                        }
                        foreach ($values as $value) {
                            add_post_meta($new_post_id, $key, maybe_unserialize($value));
                        }
                    }
                }
            }
            
        }
        wp_die();
    }

    function duplicate_terms() {
        $this->check_permission();
        em_duplicate_terms(event_m_get_param('ids'), event_m_get_param('tax_type'));
    }

    function save_global_settings() {
        $this->check_permission();
        $model = $this->request->map_request_to_model('EventM_Global_Settings_Model');
        $model= apply_filters('event_magic_gs_before_save',$model); 
        $this->services['setting']->save($model);
        do_action('em_after_gs_save');
        wp_send_json_success();
    }

    function save_booking() {
        $booking_id = absint($this->request->get_param('id'));
        $booking= $this->services['booking']->load_model_from_db($booking_id);
        $note= sanitize_text_field($this->request->get_param('note'));
        if(!empty($note)){
            $booking->notes[]= $note;
        }

        $booking= $this->services['booking']->save($booking);
        wp_send_json_success(array('post'=>$booking));
    }

    function cancel_booking() {
        $service = EventM_Factory::get_service('EventM_Booking_Service');
        $post_id = event_m_get_param('id', true);
        $response = $service->refund_booking($post_id);
        echo json_encode($response);
        die;
    }

    function load_event_dates() {
        // Get all the events dates
        $data = new stdClass();
        $data->start_dates = array();
        $data->event_ids = array();

        $event_dao = new EventM_Event_DAO();
        $events = $event_dao->get_upcoming_events_calendar();

        $event_service = EventM_Factory::get_service('EventM_Service');

        if(is_array($events)){
            foreach ($events as $event){
                $start_date = date('Y-m-d', em_get_post_meta($event->ID, 'start_date', true));
                $end_date = date('Y-m-d', em_get_post_meta($event->ID, 'end_date', true));

                if (!empty($start_date)){
                    preg_match('/[0-9]{4}\-[0-9]{1,2}-[0-9]{1,2}/', $start_date, $matches);
                    if (count($matches) > 0 && !empty($matches[0])) {

                        if (strtotime($matches[0]) <= strtotime(date('Y-m-d')) &&
                                strtotime($end_date) >= strtotime(date('Y-m-d'))):
                            $data->start_dates[] = date("Y-m-d");

                        else:
                            $data->start_dates[] = $matches[0];
                        endif;

                        $data->event_ids[] = $event->ID;
                    }

                }
            }
        }

        echo json_encode($data);
        die;
    }

    function load_venue_addresses() {
        $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
        $ids= sanitize_text_field($_POST['venue_id']);
        $mapData['address'] = $mapData['zoom_level'] = '';
        $addresses = $zoomLevels = array();
        if(!empty($ids)){
            $ids= explode(',', $ids);
            foreach($ids as $venue_id){
                $venue= $venue_service->load_model_from_db($venue_id);
                if(!empty($venue->id) && !empty($venue->address)){
                    array_push($addresses,$venue->address);
                    if(isset($venue->zoom_level) && !empty($venue->zoom_level)){
                        array_push($zoomLevels,$venue->zoom_level);
                    }
                }
            }
        }
        else{
            $venues= $venue_service->get_venues();
            foreach($venues as $venue){
                if(!empty($venue->id) && !empty($venue->address)){
                    array_push($addresses,$venue->address);
                    if(isset($venue->zoom_level) && !empty($venue->zoom_level)){
                        array_push($zoomLevels,$venue->zoom_level);
                    }
                }
            }
        }
        $mapData['address'] = $addresses;
        $mapData['zoom_level'] = $zoomLevels;
        echo json_encode($mapData);
        die;
    }

    function register_user() {
        $user_service = EventM_Factory::get_service('EventM_User_Service');
        $user_service->register_user();
    }

    function login_user() {
        $user_service = EventM_Factory::get_service('EventM_User_Service');
        $user_service->login_user();
    }

    function book_seat() {
        $data = $this->services['booking']->book_seat();
        echo json_encode($data);
        die;
    }

    public function load_payment_configuration() {
        $response = new stdClass();
        $response->payment_prcoessor = array();
        $response->is_payment_configured = em_is_payment_gateway_enabled();
        $event_service= $this->services['event'];
        $setting = $this->services['setting']->load_model_from_db();
        $event_id = $this->request->get_param('event_id');
        $event= $event_service->load_model_from_db($event_id);
        if(empty($event->id)){
            die;
        }
        
        $response->event_id = $event->id;
        $response->ticket_price = $event->ticket_price;
        $response->currency_symbol = em_currency_symbol();
        if (!empty($setting->paypal_processor)) {
            $paypal_email = $setting->paypal_email;
            if (!empty($paypal_email)) {
                $response->payment_prcoessor['paypal'] = array();
            }
        }
        $response = apply_filters('event_magic_load_payment_configuration', $response);
        if (!count($response->payment_prcoessor))
            $response->is_payment_configured = false;
        echo json_encode($response);
        die;
    }

    public function update_booking() {
        $response = new stdClass();
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        $event_service = EventM_Factory::get_service('EventM_Service');
        $order_id = event_m_get_param("order_id", true);
        $event_id = event_m_get_param("event_id", true);
        if (empty($order_id)) {
            $response->updated = false;
        }

        $order_info = em_get_post_meta($order_id, 'order_info', true);

        if (!empty($order_info)) {
            $order_info['quantity'] = event_m_get_param('quantity');
            $order_info['discount'] = event_m_get_param('discount');

            // Removing previous quantity data
            $prev_order_info = em_get_post_meta($order_id, 'order_info', true);
            //$pre_booked_seats = (int) em_get_post_meta($event_id, 'booked_seats', true);
            $pre_booked_seats = $event_service->booked_seats($event_id);
            $event_service->update_booked_seats($event_id, $pre_booked_seats - $prev_order_info['quantity']);


            if (!$booking_service->check_booking_availability($event_id, $order_info)) {
                // Removing quantity from order info as well
                $order_info['quantity'] = 0;
                em_update_post_meta($order_id, 'order_info', $order_info);
                $error = new WP_Error('error_capacity', __("Booking can't be done as no seats are available.", 'eventprime-event-calendar-management'));
                echo json_encode($error);
                die;
            }
            $current_booked_seats = $pre_booked_seats - $prev_order_info['quantity'];
            $event_service->update_booked_seats($event_id, $current_booked_seats + $order_info['quantity']);


            em_update_post_meta($order_id, 'order_info', $order_info);
            $response->updated = true;
        }

        echo json_encode($response);
        die;
    }

    public function print_ticket() {
        $booking_id = absint(event_m_get_param('booking_id'));
        $seat_no = event_m_get_param('seat_no');
        $booking = $this->services['booking']->load_model_from_db($booking_id);
        if(empty($booking->id)){
            _e("No Booking with such ID",'eventprime-event-calendar-management');
            wp_die();
        }
        $ticket_html = EventM_Print::front_ticket($booking,'',$seat_no);
        echo $ticket_html;
        die;
    }

    public function download_booking_details() {
        $booking_id = absint(event_m_get_param('booking_id'));
        $booking = $this->services['booking']->load_model_from_db($booking_id);
        if(!empty($booking->id)){
            EventM_Print::details($booking);
        }
        die;
    }

    public function event_details() {
        $response = new stdClass();
        $event_id = event_m_get_param('element_id');
        $id = get_post($event_id);
        $response->title = $id->post_title;

        $terms = wp_get_post_terms($id->ID, EM_VENUE_TYPE_TAX);
        if (!empty($terms) && count($terms) > 0):
            $venue = $terms[0];
            $venue_address = em_get_term_meta($venue->term_id, 'address', true);
            $response->address = $venue_address;
        endif;

        $booking_seats = EventM_Factory::get_service('EventM_Booking_Service');
        $my_querys = $booking_seats->get_seats($event_id);

        foreach ($my_querys as $data):
            $id = $data->ID;
            $seat = get_post_meta($id, 'em_order_info');
            foreach ($seat as $data):
                $seat_sequence = $data['seat_sequences'];
                $s = implode(',', $seat_sequence);
                $response->seats = $s;
            endforeach;

            $dates = get_post($id, 'post_date');
            $response->booking_date = $dates->post_date;
        endforeach;

        echo json_encode($response);
        die;
    }

    public function show_booking_details() {
        $user = wp_get_current_user();
        if(empty($user))
            return;
        
        $booking_id = absint(event_m_get_param('id'));
        $booking= $this->services['booking']->load_model_from_db($booking_id);
        if (empty($booking->id))
            die("No such booking exists");
        
        // Check if this booking belongs to the same user
        if ($booking->user != $user->ID)
            die("User not authorized");
        
        $event= $this->services['event']->load_model_from_db($booking->event);
        if(!empty($event->venue)){
            $venue= $this->services['venue']->load_model_from_db($event->venue);
            if(empty($venue->id)){
                $venue= null;
            }
        }
        // Load view file
        include_once('templates/booking_details.php');
        die;
    }

    public function cancel_booking_by_user() {
        $response = new stdClass();
        $response->error = true;
        
        if (is_user_logged_in()) {
            $user = wp_get_current_user();
            $booking_id = event_m_get_param('post_id', true);
            $booking = $this->services['booking']->load_model_from_db($booking_id);
            
            if (empty($booking->id))
                die("No such booking exists");
            
            // Booking can not be refunded.
            if ($booking->status == 'cancelled' || $booking->status == 'refunded') {
                echo json_encode(array());
                return;
            }

            // Check if this booking belongs to the same user
            if ($user->ID != $booking->user)
                die("User not authorized");
            
            $this->services['booking']->revoke_seats($booking_id);

            // Changing booking status
            $booking->status= 'cancelled';
            $booking= $this->services['booking']->save($booking);
            $order_info = $booking->order_info;
            if (!empty($booking)) {
                $response->error = false;
                $response->status = EventM_Constants::$status[$booking->status];
            }
            do_action('event_magic_booking_cancelled',$booking);
        }
        echo json_encode($response);
        die;
    }

    public function get_venue_capcity() {
        $response = new stdClass();
        $venue_id = event_m_get_param('venue_id');
        $event_id = event_m_get_param('event_id');

        $service = EventM_Factory::get_service('EventM_Venue_Service');
        $response->capacity = (int) $service->capacity($venue_id);
        $response->seats = $service->get_seats($venue_id, $event_id);

        echo json_encode($response);
        die;
    }

    public function verify_booking() {
        $booking_id = event_m_get_param('booking_id', true);
        $booking = get_post($booking_id);
        if (empty($booking)){
            wp_send_json_error();
        }
        else{
            $data['gateway'] = event_m_get_param("gateway", true);
            $em = event_magic_instance();
            if(in_array('coupons', $em->extensions)){
                $data['coupon_code'] = event_m_get_param("coupon_code", true);
                if(!empty($data['gateway']) && $data['gateway'] == 'paypal' && !empty($data['coupon_code'])){
                    $data['coupon_discount'] = event_m_get_param("coupon_discount", true);
                    $data['coupon_amount'] = event_m_get_param("coupon_amount", true);
                    $data['coupon_type'] = event_m_get_param("coupon_type", true);
                    $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
                    $booking = $booking_service->load_model_from_db($booking_id);
                    $booking->order_info['coupon_code'] = $data['coupon_code'];
                    $booking->order_info['coupon_discount'] = $data['coupon_discount'];
                    $booking->order_info['coupon_amount'] = $data['coupon_amount'];
                    $booking->order_info['coupon_type'] = $data['coupon_type'];
                    $bookingDao = new EventM_Booking_DAO();
                    $booking = $bookingDao->save($booking);
                }
            }
            // custom extension action on paypal data verification
            if(!empty($data['gateway']) && $data['gateway'] == 'paypal'){
                $all_order_data = $_POST['all_order_data'];
                $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
                $booking = $booking_service->load_model_from_db($booking_id);
                $booking = apply_filters('event_magic_paypal_data_verify_booking',$booking, $all_order_data);
                $bookingDao = new EventM_Booking_DAO();
                $booking = $bookingDao->save($booking);
            }
            wp_send_json_success();
        }
    }

    public function export_bookings() {
        $selections= event_m_get_param('selections');
        $bookings= array();
        if (!empty($selections)) {
            foreach ($selections as $id){
                array_push($bookings,$this->services['booking']->load_model_from_db($id));
            }
        }
        else
        {
            $bookings = $this->services['booking']->export_data();
        }
        
        $csv= new stdClass();
        foreach ($bookings as $booking) {
            $user = get_user_by('id', $booking->user);
            $csv= new stdClass();
            $csv->ID= $booking->id;
            $csv->user_display_name = $user->display_name;
            $csv->user_email = $user->user_email;
            $other_order_info = $booking->order_info;
            $csv->price =  $other_order_info['item_price'];
            $csv->no_tickets =  $other_order_info['quantity'];
            $csv->amount_total =  $other_order_info['quantity'] *  $other_order_info['item_price'];
            $event= $this->services['event']->load_model_from_db($booking->event);
            if(!empty($event->id)){
                $csv->event_name = $event->name;
            }
            else
            {
                $csv->event_name = __('Event deleted','eventprime-event-calendar-management');
            }
            $csv->status= $booking->status;
            $data->posts[] = $csv;
        }
        
        header('Content-Type: text/csv');
        header('Content-Disposition: attachment;filename="export.csv"');
        header('Cache-Control: max-age=0');
        $csv_name = 'em_Bookings' . time() . mt_rand(10, 1000000);
        $csv_path = get_temp_dir() . $csv_name . '.csv';
        $csv = fopen('php://output', "w");

        if (!$csv) {
            return false;
        }

        //Add UTF-8 header for proper encoding of the file

        fputs($csv, chr(0xEF) . chr(0xBB) . chr(0xBF));

        $csv_fields = array();

        $csv_fields[] = 'Booking%20ID';
        $csv_fields[] = 'User%20Name';
        $csv_fields[] = 'Email';
        $csv_fields[] = 'Price';
        $csv_fields[] = 'Ticket%20Count';
        $csv_fields[] = 'Total%20Amount';
        $csv_fields[] = 'Event%20Name';
        $csv_fields[] = 'Status';


        fputcsv($csv, $csv_fields);

        foreach ($data->posts as $a) {
            //print_r(array_values((array)$a));
            if (!fputcsv($csv, array_values((array) $a)))
                return false;
        }

        fclose($csv);
        wp_die();
    }

    public function check_bookable() {
        $event_id = event_m_get_param('event_id', true);
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event = $event_service->load_model_from_db($event_id);

        if ($event->status == 'expired') {
            $error = new WP_Error('booking_expired', 'Booking expired.');
            echo json_encode($error);
        }

        if((isset($event->parent) && !empty($event->parent)) && (isset($event->enable_recurrence_automatic_booking) && !empty($event->enable_recurrence_automatic_booking))){
            $error = new WP_Error('booking_not_allowed', 'Booking not allowed.');
            echo json_encode($error);
        }
        
        $available_seats = $event_service->available_seats($event_id);
        if ($available_seats <= 0) {
            $error = new WP_Error('booking_finished', __('All the seats are booked.', 'eventprime-event-calendar-management'));
            echo json_encode($error);
        }
        wp_die();
    }

    public function confirm_booking_without_payment() {
        $id = $this->request->get_param('booking_id', true);
        if(is_array($id) && !empty($id)){
            $id= $id[0];
        }
        $booking= $this->services['booking']->load_model_from_db($id);
        if(empty($booking->id)){
            wp_send_json_error();
        }
        $data['payment_gateway'] = 'none';
        $data['payment_status'] = 'completed';
        $em = event_magic_instance();
        if(in_array('coupons', $em->extensions)){
            $data['coupon_code'] = $this->request->get_param('coupon_code', true);
            $data['coupon_discount'] = $this->request->get_param('coupon_discount', true);
            $data['coupon_amount'] = $this->request->get_param('coupon_amount', true);
            $data['coupon_type'] = $this->request->get_param('coupon_type', true);
        }
        $data = apply_filters('event_magic_add_without_payment_response', $data, $this->request->get_param('all_order_data', true));
        $this->services['booking']->confirm_booking($id, $data);
        do_action('event_magic_booking_confirmed',$booking);
        $response->redirect = add_query_arg(array('em_bookings'=>$id), get_permalink(em_global_settings('profile_page')));
        if(!is_user_logged_in()){
            $showBookNowForGuestUsers = em_show_book_now_for_guest_users();
            $gs_service = EventM_Factory::get_service('EventM_Setting_Service');
            $gs = $gs_service->load_model_from_db();
            if(!empty($showBookNowForGuestUsers)){
                $response->guest_booking = 1;
                if(!empty($gs->guest_booking_page_redirect)){
                    $response->redirect = get_permalink($gs->guest_booking_page_redirect);
                }
            }
        }
        echo json_encode($response);
        wp_die();
    }

    public function rm_custom_datas() {
        $post_id = event_m_get_param('post_id');
        $user_id = em_get_post_meta($post_id, 'user', true);
        if(empty($user_id)){
            wp_die(); 
        }
        if (is_registration_magic_active()) {
            $html = "";

        echo em_rm_custom_data($user_id);
        } else {
            $current_user = get_user_by('ID', $user_id);
            ?>
            <div class="em-booking-row"><span class="em-booking-label"><?php _e('Name','eventprime-event-calendar-management'); ?>:</span><span class="em-booking-detail"><?php echo $current_user->display_name; ?></span></div>
            <div class="em-booking-row"><span class="em-booking-label"><?php _e('Registered On','eventprime-event-calendar-management'); ?>:</span><span class="em-booking-detail"><?php echo $current_user->user_registered; ?></span></div>
            <?php
        }
        wp_die();
    }

    public function resend_mail() {
        $this->check_permission();
        $booking_id = event_m_get_param('post_id');
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        $booking= $booking_service->load_model_from_db($booking_id);
        $user= get_user_by('ID',$booking->user);
        if(empty($user))
            return false;
        $new_user_password = wp_generate_password(5);
        wp_set_password($new_user_password,$user->ID);
        EventM_Notification_Service::reset_password_mail($booking,$new_user_password);
        die;
    }

    public function booking_cancellation_mail() {
        $booking_id = event_m_get_param('post_id');
        EventM_Notification_Service::booking_cancel($booking_id);
    }

    public function booking_confirm_mail() {
        $booking_id = event_m_get_param('post_id');
        EventM_Notification_Service::booking_confirmed($booking_id);
    }

    public function booking_refund_mail() {
        $booking_id = event_m_get_param('post_id');
        EventM_Notification_Service::booking_refund($booking_id);
    }

    public function booking_pending_mail() {
        $booking_id = event_m_get_param('post_id');
        EventM_Notification_Service::booking_pending($booking_id);
    }
    
    public function event_submitted_mail() {
        $event_id = absint(event_m_get_param('post_id'));
        EventM_Notification_Service::event_submitted($event_id);
        wp_send_json_success();
    }
    
    public function event_approved_mail() {
        $event_id = absint(event_m_get_param('id'));
        EventM_Notification_Service::event_approved($event_id);
        wp_send_json_success();
    }

    public function load_event_for_booking() {
        $em = event_magic_instance();
        if (!is_user_logged_in() && (!in_array('guest-booking', $em->extensions) || empty(em_global_settings('allow_guest_bookings')))){
            wp_die("User not logged in");
        }

        // Event's data
        $event_id = $this->request->get_param('event_id', true);
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event = $event_service->load_model_from_db($event_id);
        $event->available_seats = $event_service->available_seats($event_id);
        $event->start_date= em_showDateTime($event->start_date, true);
        // Venue's data
        $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
        $venue = $venue_service->load_model_from_db($event->venue);
        $venue->available_seats = $event_service->available_seats($event_id);
        // Unset seats info as it is already included in event's data
        unset($venue->seats);
        // Mergin both event and venue data
        $event->venue = $venue;
        // check for automatic discount
        $event->allow_automatic_discounts = 0;
        if(in_array('em_automatic_discounts', $em->extensions) && !empty(em_global_settings('allow_early_bird_discount'))){
            $event->allow_automatic_discounts = 1;
        }

        echo json_encode($event);
        wp_die();
    }

    function event_tour_completed() {
        $global_options = get_option(EM_GLOBAL_SETTINGS);
        $global_options['event_tour'] = 1;
        update_option(EM_GLOBAL_SETTINGS, $global_options);
        wp_die();
    }

    function add_event_tour_completed() {
        $global_options = get_option(EM_GLOBAL_SETTINGS);
        $global_options['add_event_tour'] = 1;
        update_option(EM_GLOBAL_SETTINGS, $global_options);
        wp_die();
    }

    function delete_order() {
        $order_id = $this->request->get_param('order_id', true);
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        $booking_service->revoke_seats($order_id, 'tmp', 'general');
        wp_delete_post($order_id);
        exit(0);
    }
    
    public function deactivation_feedback() {
        $msg = event_m_get_param('msg');
        $feedback = event_m_get_param('feedback');
        $message= '';
        $from_email_address = em_get_admin_user_email();
        switch($feedback) {
            case 'feature_not_available': $body='Feature not available: '; break;
            case 'feature_not_working': $body='Feature not working: '; break;
            case 'found_a_better_plugin': $body='Found a better plugin: '; break;
            case 'plugin_broke_site': $body='Plugin broke my site'; break;
            case 'plugin_stopped_working': $body='Plugin stopped working'; break;
            case 'temporary_deactivation': $body = "It's a temporary deactivation"; break;
            case 'other': $body='Other: '; break;
            default: return;
        }
        if(!empty($feedback)) {
            $message .= $body."\n\r";
            if(!empty($msg)) {
                $message .= $msg."\n\r";
            }
            $message .= "\n\r EventPrime Version - ".EVENTPRIME_VERSION;
            $headers = "MIME-Version: 1.0\r\n";
            $headers .= "Content-type:text/html;charset=UTF-8\r\n";
            $headers .= 'From:'.$from_email_address."\r\n";
            if (wp_mail('feedback@cmshelplive.com','EventPrime Feedback',$message,$headers))
                wp_send_json_success();
            else
                wp_send_json_error();
        }
    }
    
    public function frontend_event_submit_form_strings() {
        $response = $this->services['event']->load_frontend_event_submit_page();
        wp_send_json_success(array('settings'=>$response));
    }
    
    public function submit_frontend_event() {
        $errors = array();
        
        // Event name validation
        $event_name = sanitize_text_field(event_m_get_param('name'));
        if (empty($event_name)) {
            $errors[]= __('Event Name cannot be empty.','eventprime-event-calendar-management');
        }
            
        // Dates validation
        $start_date = em_timestamp(event_m_get_param('start_date'));
        $end_date = em_timestamp(event_m_get_param('end_date'));
        if ($start_date > $end_date) {
            $errors[]= __('Event Start Date should be prior to the End Date.','eventprime-event-calendar-management');
        }
        
        $enable_booking = absint(event_m_get_param('enable_booking'));
        if (!empty($enable_booking)){
            $start_booking_date =  em_timestamp(event_m_get_param('start_booking_date'));
            $last_booking_date = em_timestamp(event_m_get_param('last_booking_date'));
            if (!empty($start_booking_date) && !empty($last_booking_date)) {
                if ($last_booking_date > $end_date) {
                    $errors[]=__('Last Booking Date can not be greater than Event End Date.','eventprime-event-calendar-management');
                }
                if ($start_booking_date > $last_booking_date) {
                    $errors[]=__('Start Booking Date should be earlier than the Last Booking Date.','eventprime-event-calendar-management');
                }
                if ($start_booking_date > $start_date) {
                    $errors[]=__('Start Booking Date must be earlier than the Event Start Date.','eventprime-event-calendar-management');
                }
                if ($start_booking_date == $last_booking_date) {
                    $errors[]=__('Start Booking Date must be earlier than the Last Booking Date.','eventprime-event-calendar-management');
                }
            } else {
                $errors[]=__('Start Booking Date and Last Booking Date cannot be empty','eventprime-event-calendar-management');
            }
        }
        /*
        if (absint(event_m_get_param('custom_link_enabled')) == 1){
            if(!preg_match('/^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/',event_m_get_param('custom_link'))){
                $errors[]=__('Incorrect custom link. Please update custom link to a correct URL format.','eventprime-event-calendar-management');
            }
        }
        */
        if(!empty($errors)){
            wp_send_json_error(array('errors'=>$errors));
        }
        
        $response = $this->services['event']->save_user_submitted_event();
        wp_send_json_success(array('post_id'=>$response));
    }
    
    private function check_permission(){
        if(!em_is_user_admin()){
            $error_msg= __('User not allowed','eventprime-event-calendar-management');
            wp_send_json_error(array('errors'=>array($error_msg)));
        }
    }

    public function upload_image_from_frontend(){
        if(isset($_FILES["image"]) && !empty($_FILES["image"])){
            $extension = pathinfo($_FILES["image"]["name"], PATHINFO_EXTENSION);
            if($extension !='jpg' && $extension !='jpeg' && $extension != 'png' && $extension != 'gif'){
                wp_send_json_error( array( 'errors' => array( 'Only Image File Allowed.' ) ) );
            }
            $file = $_FILES['image'];
            $filename = $file['name'];
            $tmp_name = $file['tmp_name'];
            $upload_dir = wp_upload_dir();
            
            if (move_uploaded_file($file["tmp_name"], $upload_dir['path'] . "/" . $filename)) {
                $uploaded_file['file_name'] = $filename;
                $uploaded_file['upload_url'] = $upload_dir['url'] . "/" . $filename;

                $wp_filetype = wp_check_filetype($filename, null );
                $attachment = array(
                    'guid'           => $uploaded_file['upload_url'],
                    'post_mime_type' => $wp_filetype['type'],
                    'post_title'     => preg_replace( '/\.[^.]+$/', '', $filename ),
                    'post_content'   => '',
                    'post_status'    => 'inherit'
                );
                $attachment_id = wp_insert_attachment( $attachment, $upload_dir['path'] . "/" . $filename );
     
                if ( ! is_wp_error( $attachment_id ) ) {
                    require_once(ABSPATH . "wp-admin" . '/includes/image.php');
                    $attachment_data = wp_generate_attachment_metadata( $attachment_id, $upload_file['file'] );
                    wp_update_attachment_metadata( $attachment_id,  $attachment_data );
                    wp_send_json_success(array('attachment_id' => $attachment_id));
                }
            }
            else{
                wp_send_json_error(array('errors'=>array($upload_file['error'])));
            }
        }
    }
}
EventM_AJAX::get_instance();