<?php

if (!defined('ABSPATH')) {
    exit;
}

class EventM_Install {

    public static function install($network_wide) {
        global $wpdb;

        if (is_multisite() && $network_wide) {
            foreach ($wpdb->get_col("SELECT blog_id FROM $wpdb->blogs") as $blog_id) {
                switch_to_blog($blog_id);
                self::create_pages();
                self::default_settings();
                restore_current_blog();
            }
        } else {
            self::create_pages();
            self::default_settings();
        }
        add_option('event_magic_do_activation_redirect', true);
    }

    private static function create_pages() {
        global $wpdb;
        $global_options = get_option(EM_GLOBAL_SETTINGS);

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_performers]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['performers_page'] = $page_id;
        } else {
            // Creating performers page
            $args = array("post_content" => "[em_performers]",
                "post_title" => "Performers",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['performers_page'] = $page_id;
            }
        }

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_sites]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['venues_page'] = $page_id;
        } else {
            // Creating Venues page
            $args = array("post_content" => "[em_sites]",
                "post_title" => "Event Sites & Locations",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['venues_page'] = $page_id;
            }
        }

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_events%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['events_page'] = $page_id;
        } else {
            // Creating Events page
            $args = array("post_content" => "[em_events]",
                "post_title" => "Events",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['events_page'] = $page_id;
            }
        }

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_booking]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['booking_page'] = $page_id;
        } else {
            // Creating Events page
            $args = array("post_content" => "[em_booking]",
                "post_title" => "Booking",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['booking_page'] = $page_id;
            }
        }

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_profile]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['profile_page'] = $page_id;
        } else {
            // Creating Events page
            $args = array("post_content" => "[em_profile]",
                "post_title" => "User Profile",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['profile_page'] = $page_id;
            }
        }

        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_event_types]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['event_types'] = $page_id;
        } else {
            // Creating Events page
            $args = array("post_content" => "[em_event_types]",
                "post_title" => "Event Types",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['event_types'] = $page_id;
            }
        }
        
        $sql = "SELECT ID FROM $wpdb->posts WHERE POST_CONTENT like '%[em_event_submit_form]%' AND POST_TYPE IN ('page','post')";
        $page_id = $wpdb->get_var($sql);
        if (!empty($page_id)) {
            $global_options['event_submit_form'] = $page_id;
        } else {
            // Creating Events page
            $args = array("post_content" => "[em_event_submit_form]",
                "post_title" => "Submit Event",
                "post_status" => "publish",
                "post_type" => "page");
            $page_id = wp_insert_post($args);
            if (!is_wp_error($page_id)) {
                $global_options['event_submit_form'] = $page_id;
            }
        }
        
        // Update settings
        update_option(EM_GLOBAL_SETTINGS, $global_options);
    }

    private static function default_settings() {
        global $wp_rewrite;
        self::default_notifications();
        $instance = event_magic_instance();

        $global_options = get_option(EM_GLOBAL_SETTINGS);

        if (!isset($global_options['payment_test_mode']))
            $global_options['payment_test_mode'] = 1;

        if (!isset($global_options['currency']))
            $global_options['currency'] = EM_DEFAULT_CURRENCY;

        if (!isset($global_options['event_tour'])) {
            $global_options['event_tour'] = 0;
        }
        if (!isset($global_options['is_visit_welcome_page'])) {
            $global_options['is_visit_welcome_page'] = 0;
        }
        if (!isset($global_options['dashboard_hide_past_events'])) {
            $global_options['dashboard_hide_past_events'] = 0;
        }
        if (!isset($global_options['disable_filter_options'])) {
            $global_options['disable_filter_options'] = 0;
        }
        // Update settings
        update_option(EM_GLOBAL_SETTINGS, $global_options);
        $wp_rewrite->flush_rules();

        // Update DB version
        update_option(EM_DB_VERSION, $instance->version);
    }

    private static function default_notifications() {
        $global_options = get_option(EM_GLOBAL_SETTINGS);

        $booking_pending_email = em_global_settings('booking_pending_email');
        if (empty($booking_pending_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/pending.html');
            $global_options['booking_pending_email'] = ob_get_clean();
        }

        $booking_confirmed_email = em_global_settings('booking_confirmed_email');
        if (empty($booking_confirmed_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/customer.html');
            $global_options['booking_confirmed_email'] = ob_get_clean();
        }

        $booking_cancelation_email = em_global_settings('booking_cancelation_email');
        if (empty($booking_cancelation_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/cancellation.html');
            $global_options['booking_cancelation_email'] = ob_get_clean();
        }

        $reset_password_mail = em_global_settings('reset_password_mail');
        if (empty($reset_password_mail)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/reset_user_password.html');
            $global_options['reset_password_mail'] = ob_get_clean();
        }

        $registration_email_content = em_global_settings('registration_email_content');
        if (empty($registration_email_content)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/registration.html');
            $global_options['registration_email_content'] = ob_get_clean();
        }

        $booking_refund_email = em_global_settings('booking_refund_email');
        if (empty($booking_refund_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/refund.html');
            $global_options['booking_refund_email'] = ob_get_clean();
        }
        
        $event_submitted_email = em_global_settings('event_submitted_email');
        if (empty($event_submitted_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_submitted.html');
            $global_options['event_submitted_email'] = ob_get_clean();
        }
        
        $event_approved_email = em_global_settings('event_approved_email');
        if (empty($event_approved_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_approved.html');
            $global_options['event_approved_email'] = ob_get_clean();
        }

        // Update settings
        update_option(EM_GLOBAL_SETTINGS, $global_options);
    }
}
