<?php

// Exit if accessed directly
if (!defined('ABSPATH'))
    exit;

if (!class_exists('EM_Shortcodes')) {

    class EM_Shortcodes {

        private static $instance = null;
        public $codes = array('em_events' => 'load_events', 'em_performers' => 'load_performers', 'em_event_types' => 'load_event_types',
            'em_sites' => 'load_venues', 'em_booking' => "load_booking", 'em_profile' => "load_profile", 'em_event' => 'load_single_event',
            'em_event_type' => 'load_single_event_type', 'em_performer' => 'load_single_performer', 'em_event_site' => 'load_single_venue',
            'em_event_submit_form' => 'load_event_submit_form');

        public static function get_instance() {
            if (self::$instance === null) {
                self::$instance = new self();
            }
            return self::$instance;
        }

        private function __construct() {
            $this->add_shortcodes();
        }

        private function add_shortcodes() {
            foreach ((array) $this->codes as $code => $function) {
                add_shortcode($code, array($this, $function));
            }
        }

        public function load_profile($atts) {
            $profile_shortcode = apply_filters('ep_load_profile',false,$atts);
            if(empty($profile_shortcode)){
                ob_start();
                include('templates/user_profile.php');
                return ob_get_clean();
            }
            else{
                return do_shortcode($profile_shortcode);
            }
        }

        public function load_single_event($atts) {
            ob_start();
            $atts = array_change_key_case((array)$atts, CASE_LOWER);
            $event_id = absint($atts['id']);
            $post = get_post($event_id);
            if (!empty($post)) {
                if ($post->post_type == "em_event" && $post->post_status != "trash") {
                    if ($post->post_status == "draft" && !current_user_can('manage_options')) {
                        _e('This event is currently in draft state', 'eventprime-event-calendar-management');
                    } else {
                        $event_id = $post->ID;
                        include('templates/event.php');
                    }
                }
            }
            return ob_get_clean();
        }

        public function load_events($atts = [], $content = null, $tag = '') {
            ob_start();
            $event_id = absint(event_m_get_param('event'));

            if (!empty($event_id)) {
                $post = get_post($event_id);
                if (!empty($post) && $post->post_type == "em_event" && $post->post_status != "trash") {
                    if ($post->post_status == "draft" && !current_user_can('manage_options')) {
                        _e('This event is currently in draft state', 'eventprime-event-calendar-management');
                    } else {
                        include('templates/event.php');
                    }
                }
            }
            else {
                $gs_service = EventM_Factory::get_service('EventM_Setting_Service');
                $global_settings = $gs_service->load_model_from_db();
                $atts = array_change_key_case((array)$atts, CASE_LOWER);
                $cal_args = [
                    'view' => $global_settings->default_cal_view,
                    'types' => array(),
                    'sites' => array()
                ];
                if($global_settings->default_cal_view == 'card'){
                    $cal_args['show'] = '';
                }
                $events_atts = shortcode_atts($cal_args, $atts, $tag);
                if (!in_array($events_atts['view'],array('card','list','month','week','day'))) {
                    $events_atts['view'] = $global_settings->default_cal_view;
                }
                if (!is_array($events_atts['types'])) {
                    $events_atts['types'] = explode(',',$events_atts['types']);
                }
                if (!is_array($events_atts['sites'])) {
                    $events_atts['sites'] = explode(',',$events_atts['sites']);
                }
                if ($events_atts['view'] != 'card') {
                    $events_atts['show'] = '';
                }
                include('templates/events.php');
            }
            return ob_get_clean();
        }

        public function load_single_performer($atts) {
            ob_start();
            $atts = array_change_key_case((array)$atts, CASE_LOWER);
            $performer_id = absint($atts['id']);
            $post = get_post($performer_id);

            if (empty($post) || $post->post_type != 'em_performer' || $post->post_status == 'trash') {
                _e('No such performer found', 'eventprime-event-calendar-management');
                return ob_get_clean();
            }

            include('templates/single-performer.php');
            return ob_get_clean();
        }

        public function load_performers($atts) {
            ob_start();
            $performer_id = absint(event_m_get_param('performer'));
            if (!empty($performer_id)) {
                $post = get_post($performer_id);
                if (!empty($post) && $post->post_type == "em_performer" && $post->post_status != "trash")
                    include('templates/single-performer.php');
            } else
                include('templates/performers.php');
            return ob_get_clean();
        }

        public function load_venues($atts) {
            ob_start();
            include('templates/venues.php');
            return ob_get_clean();
        }

        public function load_single_venue($atts) {
            ob_start();
            $atts = array_change_key_case((array)$atts, CASE_LOWER);
            $venue_page_id = em_global_settings('venues_page');
            $venue_id = absint($atts['id']);
            $service = EventM_Factory::get_service('EventM_Venue_Service');
            $venue = $service->load_model_from_db($venue_id);
            if (empty($venue->id)) {
                _e('No such site exists.', 'eventprime-event-calendar-management');
            } else {
                include_once 'templates/venue.php';
            }
            return ob_get_clean();
        }

        public function load_booking() {
            ob_start();
            if (is_user_logged_in()) { // Removing all the temporary bookings for current user
                $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
                $user = wp_get_current_user();
                $booking_service->remove_tmp_bookings_for_user($user->ID);
            }
            include('templates/booking.php');
            return ob_get_clean();
        }

        public function load_event_types($atts) {
            ob_start();
            $type_id = absint(event_m_get_param('type'));
            $event_type_service = EventM_Factory::get_service('EventTypeM_Service');
            if (!empty($type_id)) {
                $type = $event_type_service->load_model_from_db($type_id);
                if (is_wp_error($type) || empty($type) || is_null($type)) {
                    _e('No such Event Type exists', 'eventprime-event-calendar-management');
                    return ob_get_clean();
                } else {
                    include('templates/single-event-type.php');
                }
            } else {
                include('templates/event_types.php');
            }
            return ob_get_clean();
        }

        public function load_single_event_type($atts) {
            ob_start();
            $atts = array_change_key_case((array)$atts, CASE_LOWER);
            $type_id = absint($atts['id']);
            $event_type_service = EventM_Factory::get_service('EventTypeM_Service');
            if (!empty($type_id)) {
                $type = $event_type_service->load_model_from_db($type_id);
                if (is_wp_error($type) || empty($type) || is_null($type)) {
                    _e('No such Event Type exists', 'eventprime-event-calendar-management');
                    return ob_get_clean();
                } else {
                    include('templates/single-event-type.php');
                }
            } else {
                include('templates/event_types.php');
            }
            return ob_get_clean();
        }
        
        public function load_event_submit_form($atts) {
            ob_start();
            include('templates/event_submit_form.php');
            return ob_get_clean();
        }
        
    }
    
}

EM_Shortcodes::get_instance();
