<?php

if (!defined('ABSPATH')) {
    exit;
}

class EventM_Service {

    private $dao;
    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        $this->dao = new EventM_Event_DAO();
    }

    public function load_edit_page() {
        $response = new stdClass();
        $id = event_m_get_param('post_id');
        $event = $this->load_model_from_db($id);
        $event->start_date= !empty($event->start_date) ? em_showDateTime($event->start_date,true,"m/d/Y") : '';
        $event->end_date= !empty($event->end_date) ? em_showDateTime($event->end_date,true,"m/d/Y") : '';
        $event->start_booking_date= !empty($event->start_booking_date) ? em_showDateTime($event->start_booking_date,true,"m/d/Y") : '';
        $event->last_booking_date= !empty($event->last_booking_date) ? em_showDateTime($event->last_booking_date,true,"m/d/Y") : '';
        
        $response->post = $event->to_array();
        $response->post['venues'] = $this->get_venues_dropdown();
        $response->post['date_format'] = "mm/dd/yy";
        $response->post['cover_image_id'] = get_post_thumbnail_id($id);
        $response->post['cover_image_url'] = $this->get_event_cover_image($id);
        $response->post['images'] = $this->get_event_images($response->post['gallery_image_ids']);
        $response->post['sponser_images'] = $this->get_event_images($response->post['sponser_image_ids']);

        // Loading Currency symbol
        $global_settings = new EventM_Global_Settings_Model();
        $response->post['currency'] = em_currency_symbol();
        $response->post['status_list'] = em_array_to_options(array(
            "publish" => __('Active','eventprime-event-calendar-management'),
            "expired" => __('Unpublished','eventprime-event-calendar-management'),
            "draft" => __('Draft','eventprime-event-calendar-management')
        ));
        $response->post['performers'] = $this->get_performers_dropdown();
        $response->post['event_types'] = $this->get_types_dropdown();
        $response->post['ticket_templates'] = $this->get_ticket_dropdown();
        $response->rm_forms = $this->get_rm_forms();
        
        $response = apply_filters('eventprime_load_post_response', $response);
                
        return $response;
    }
    
    public function load_frontend_event_submit_page() {
        $global_settings = new EventM_Global_Settings_Model();
        $response = array();
        $response['date_format'] = "mm/dd/yy";
        $response['currency'] = em_currency_symbol();
        $response['venues'] = $this->get_venues_dropdown();
        /*foreach($response['venues'] as $id => $venue){
            if(is_string($venue->id)){
                unset($response['venues'][$id]);
            }
        }*/
        $response['performers'] = $this->get_performers_dropdown();
        foreach($response['performers'] as $id => $performer){
            if(is_string($performer->id)){
                unset($response['performers'][$id]);
            }
        }
        $response['performer'] = array();
        $response['event_types'] = $this->get_types_dropdown();
        foreach($response['event_types'] as $id => $type){
            if(is_string($type->id)){
                unset($response['event_types'][$id]);
            }
        }
        $response['enable_booking'] = 0;
        $response['ticket_price'] = 0.00;
        $response['all_day'] = 0;
        $response['user_submitted'] = 1;
        $response['user'] = get_current_user_id();
        $response['rm_forms'] = $this->get_rm_forms();
        
        $response = apply_filters('eventprime_frontend_event_submit_response', $response);

        return $response;
    }

    /*
    * Load List page for REST 
    */
    public function load_list_page() {
        $gs = EventM_Factory::get_service('EventM_Setting_Service');
        $gs_model = $gs->load_model_from_db();
        $type_service = EventM_Factory::get_service('EventTypeM_Service');
        
        $response = new stdClass();		             
        $response->posts = array();	
        $hideExpired = event_m_get_param('hideExpired');
        if ($hideExpired == true) {	
            $gs_model->hide_expired_from_admin=1;		             
        } 
        else 		
        {		
            $full_load = event_m_get_param('full_load');		
            if (!$full_load) {		
                $gs_model->hide_expired_from_admin=0;		
            }		
        }
        $gs->save($gs_model);	
        $response->hideExpired = $gs_model->hide_expired_from_admin;
        $post_status = 'any';		
        if ($gs_model->hide_expired_from_admin == 1) {		
            $post_status = array('publish','draft');		
        }
        
        $args = array(		
              'posts_per_page' => EM_PAGINATION_LIMIT,		
              'offset' => ((int) event_m_get_param('paged') - 1) * EM_PAGINATION_LIMIT,	
              'orderby'=>'date', 
              'order' => 'DESC',		
              'post_type' => 'em_event',		
              'post_status' => $post_status
            );		
  		
        $events = $this->dao->get_events($args);
        foreach ($events as $p) {		
            $post = new stdClass();		
            $event = $this->load_model_from_db($p->ID);		
            $post->sum = $this->booked_seats($event->id);		
            $post->id = $event->id;		
            $post->name = $event->name;		
            $post->is_expired = em_is_event_expired($event->id);		
            $venue_terms = wp_get_object_terms($event->id, EM_EVENT_VENUE_TAX);		

            if (!empty($venue_terms))		
                $post->venue_name = $venue_terms[0]->name;		

            $post->capacity = em_event_seating_capcity($event->id);		
            $post->between = em_showDateTime($event->start_date,true,"m/d/Y") . " to " . em_showDateTime($event->end_date,true,"m/d/Y");		
            // Cover Image 		
            $cover_image_id = get_post_thumbnail_id($event->id);		
            if (!empty($cover_image_id) && $cover_image_id > 0) {		
                $cover_image = wp_get_attachment_image_src($cover_image_id, 'large');		
                $post->cover_image_url = $cover_image[0];		
            }		
            $response->posts[] = $post;		
        }
        
        $response->trans = new stdClass();				
        $response->total_posts = range(1, $this->get_total_events($args));		
        $response->event_ids = array();		
        $events = $this->dao->get_events($args);
        foreach ($events as $p) {		
            $response->event_ids[] = $p->ID;
        }

        $response->trans->manager_navs = em_manager_navs();
        $response->manager_nav = 'event';		
        $response->pagination_limit = EM_PAGINATION_LIMIT;
        
        $response->performers= EventM_Factory::get_service('EventM_Performer_Service')->get_all();
        $temp_venues= EventM_Factory::get_service('EventM_Venue_Service')->get_venues();
        $venues= array();
        foreach($temp_venues as $index=>$v){
            array_push($venues,array('id'=>$v->id,'name'=>$v->name));
        }
        $response->venues[]= array('id'=>0,'name'=>__('Select Event Site'));
        $response->venues= array_merge($response->venues,$venues);
        if(empty($venues)){
            $response->venues= array();
        }
        $types = $type_service->get_types();
        $event_types = array();
        $status_list = em_array_to_options(array(
            "publish" => __('Active','eventprime-event-calendar-management'),
            "expired" => __('Unpublished','eventprime-event-calendar-management'),
            "draft" => __('Draft','eventprime-event-calendar-management')
        ));
        $colors= array();
        if(!empty($types)){
            // Insert default value 
            $tmp = new stdClass();
            $tmp->id = 0;
            $tmp->name = __('Select Event Type', 'eventprime-event-calendar-management');
            $event_types[] = $tmp;
            foreach ($types as $type) {
                $tmp = new stdClass();
                $tmp->name = $type->name;
                $tmp->id = $type->id;
                $colors[$type->id]= $type->color;
                $event_types[] = $tmp;
            }
        }
        $response->event_types=$event_types;
        $response->colors=$colors;
        $response->status_list=$status_list;
        
        return $response;
    }
    
    public function admin_calendar_view() {
        $data= array('events'=>array());
        $events= $this->get_all();
        $type_service = EventM_Factory::get_service('EventTypeM_Service');
        foreach($events as $ev){
            $event= array();
            $event['id']= $ev->id;
            $event['title']= htmlspecialchars_decode($ev->name);
            $event['start']= date('c',$ev->start_date);
            $event['end']= date('c',$ev->end_date);
            $event['enable_booking']= (isset($ev->enable_booking) && !empty($ev->enable_booking)) ? absint($ev->enable_booking) : 0;
            $event['ticket_price']= (isset($ev->ticket_price) && !empty($ev->ticket_price)) ? abs(floatval($ev->ticket_price)) : 0.00;
            $event['start_booking_date']= (isset($ev->start_booking_date) && !empty($ev->start_booking_date)) ? $ev->start_booking_date : '';
            $event['last_booking_date']= (isset($ev->last_booking_date) && !empty($ev->last_booking_date)) ? $ev->last_booking_date : '';
            $event['performer']= $ev->performer;
            $event['venue']= $ev->venue;
            $event['event_type']= $ev->event_type;
            $event['all_day']= $ev->all_day;
            $event['status']= $ev->status;
            $event['popup']= $this->admin_event_hover_popup($ev);
            if (!empty($ev->event_type)) {
                $type_model = $type_service->load_model_from_db($ev->event_type);
                $event['bg_color'] = '#' . $type_model->color;
            }
            if(!empty($ev->event_text_color)){
                $event['event_text_color'] = '#' . $ev->event_text_color;
            }
            $event = apply_filters('ep_admin_calendar_event',$event,$ev);
            array_push($data['events'],$event);
        }
        $data['performers']= EventM_Factory::get_service('EventM_Performer_Service')->get_all();
        $temp_venues= EventM_Factory::get_service('EventM_Venue_Service')->get_venues();
        $venues= array();
        foreach($temp_venues as $index=>$v){
            array_push($venues,array('id'=>$v->id,'name'=>$v->name));
        }
        $data['venues'][]= array('id'=>0,'name'=>__('Select Event Site'));
        $data['venues']= array_merge($data['venues'],$venues);
        if(empty($venues)){
            $data['venues']= array();
        }
        $types = $type_service->get_types();
        $event_types = array();
        $status_list = em_array_to_options(array(
            "publish" => __('Active','eventprime-event-calendar-management'),
            "expired" => __('Unpublished','eventprime-event-calendar-management'),
            "draft" => __('Draft','eventprime-event-calendar-management')
        ));
        $colors= array();
        if(!empty($types)){
            // Insert default value 
            $tmp = new stdClass();
            $tmp->id = 0;
            $tmp->name = __('Select Event Type', 'eventprime-event-calendar-management');
            $event_types[] = $tmp;
            foreach ($types as $type) {
                $tmp = new stdClass();
                $tmp->name = $type->name;
                $tmp->id = $type->id;
                $colors[$type->id]= $type->color;
                $event_types[] = $tmp;
            }
        }
        $data['event_types']=$event_types;
        $data['status_list']=$status_list;
        $data['colors']=$colors;
        wp_send_json_success($data);
    }

    public function get_total_events($args = array()) {
        if (isset($args['posts_per_page']))
            unset($args['posts_per_page']);
        $args['numberposts'] = -1;
        $posts = get_posts($args);
        $total_count = count($posts);
        return $total_count;
    }
    
    public function get_all($args= array()) {
        $posts = $this->dao->get_events($args);
        $events= array();
        foreach($posts as $post){
            $event = $this->load_model_from_db($post->ID);
            array_push($events,$event);
        }
        return $events;
    }

    public function get_event_cover_image($id, $size = 'thumbnail') {
        $cover_image_url = get_the_post_thumbnail_url($id, $size);
        if ($cover_image_url === false)
            return "";
        else
            return $cover_image_url;
    }

    public function get_event_images($image_ids) {
        //  Gallery Image Ids
        $gallery_image_ids = maybe_unserialize($image_ids);
        $images = array();

        if (!empty($gallery_image_ids) && $gallery_image_ids != "") {
            $image_ids = array_unique($gallery_image_ids);
            foreach ($image_ids as $image_id) {
                $tmp = new stdClass();
                $tmp->src = wp_get_attachment_image_src($image_id);
                $tmp->id = $image_id;
                $images[] = $tmp;
            }
        }

        return $images;
    }

    public function get_rm_forms() {
        $rm_forms = array();
        // Registration Magic Integration
        if (is_registration_magic_active()) {
            $where = array("form_type" => 1);
            $data_specifier = array('%d');
            $forms = RM_DBManager::get('FORMS', $where, $data_specifier, 'results', 0, 99999, '*', $sort_by = 'created_on', $descending = true);
            $form_dropdown_array = array();
            $form_dropdown_array[0] = __('Default EventPrime Form','eventprime-event-calendar-management');
            if ($forms)
                foreach ($forms as $form)
                    $form_dropdown_array[$form->form_id] = $form->form_name;
            $rm_forms = $form_dropdown_array;
        }
        return $rm_forms;
    }

    public function save_settings($event){
        $event->name= sanitize_text_field(event_m_get_param('name'));
        $event->description= wp_kses_post(stripslashes(event_m_get_param('description')));
        
        // Event Type handling
        $event_type = sanitize_text_field(event_m_get_param('event_type'));
        if($event_type=='new_event_type'){ // If admin chosen to add new Event Type
            $new_event_type_name = sanitize_text_field(event_m_get_param('new_event_type'));
            $new_event_type_color = sanitize_text_field(event_m_get_param('new_event_type_color'));
            $type_service = EventM_Factory::get_service('EventTypeM_Service');
            //Creating Type model
            $type= new EventM_Event_Type_Model();
            $type->name=$new_event_type_name;
            $type->color=$new_event_type_color;
            $type= $type_service->save($type);
            $this->dao->set_type($event->id,$type->id);
            $event->event_type=$type->id;
        }
        else{
            $event->event_type= absint($event_type);
        }
        
        // Feature image handling
        $cover_image_id = absint(event_m_get_param('cover_image_id'));
        if (!empty($cover_image_id)) {
            $this->dao->set_thumbnail($event->id,$cover_image_id);
            $event->cover_image_id=$cover_image_id;
        }
        else{ // If cover image not uploaded then set first image from gallery
            $gallery_image_ids=event_m_get_param('gallery_image_ids');
            if (is_array($event->gallery_image_ids)) {
                if(!empty($gallery_image_ids[0])){
                    $img_id= absint($gallery_image_ids[0]);
                    $this->dao->set_thumbnail($event->id,$img_id);
                    $event->cover_image_id=$img_id;
                }
            }
        }
        
        // Gallery images handling
        $gallery_image_ids=event_m_get_param('gallery_image_ids');
        if(is_array($gallery_image_ids)){
            $event->gallery_image_ids=$gallery_image_ids;
        }
        // Event Dates
        $start_date= sanitize_text_field(event_m_get_param('start_date'));
        $event->start_date= !empty($start_date) ? em_timestamp($start_date) : '';
        $end_date= sanitize_text_field(event_m_get_param('end_date'));
        $event->end_date= !empty($end_date) ? em_timestamp($end_date) : '';
        $start_booking_date= sanitize_text_field(event_m_get_param('start_booking_date'));
        $event->start_booking_date= !empty($start_booking_date) ? em_timestamp($start_booking_date) : $event->start_date;
        $last_booking_date= sanitize_text_field(event_m_get_param('last_booking_date'));
        $event->last_booking_date= !empty($last_booking_date) ? em_timestamp($last_booking_date) : $event->end_date;
        
        $event->enable_booking= absint(event_m_get_param('enable_booking'));
        $event->hide_booking_status=absint(event_m_get_param('hide_booking_status'));
        $event->allow_cancellations=absint(event_m_get_param('allow_cancellations'));
        $event->enable_attendees=absint(event_m_get_param('enable_attendees'));
        $event->show_attendees=absint(event_m_get_param('show_attendees'));
        $event->rm_form= absint(event_m_get_param('rm_form'));
        $event->all_day= absint(event_m_get_param('all_day'));
        $event->hide_event_from_calendar= absint(event_m_get_param('hide_event_from_calendar'));
        $event->hide_event_from_events= absint(event_m_get_param('hide_event_from_events'));
        $event->audience_notice= wp_kses_post(stripslashes(event_m_get_param('audience_notice')));
        $event->status= event_m_get_param('status');
        $event->ticket_price= (float) event_m_get_param('ticket_price');
        $event->custom_link_enabled= absint(event_m_get_param('custom_link_enabled'));
        $event->custom_link= event_m_get_param('custom_link');
        $event->event_text_color = sanitize_text_field(event_m_get_param('event_text_color'));
        $event_id = $this->dao->save($event);
        return $event_id;
    }
    
    public function save_performers($event){
        $performer = event_m_get_param('performer');
        if ($performer[0]=='new_performer') { // Admin chosen to add new performer
            $performer_service = EventM_Factory::get_service('EventM_Performer_Service');
            // Check if multiple performers given seperated by comma
            $performer_name= sanitize_text_field(event_m_get_param('custom_performer_name'));
            $performers = explode(',',$performer_name);
            $performer_ids = array();
            foreach ($performers as $performer_name){
                if (!empty($performer_name)){
                    $model = new EventM_Performer_Model();
                    $model->name = $performer_name;
                    $type = sanitize_text_field(event_m_get_param('custom_performer_type'));
                    $model->type= empty($type) ? 'person' : $type;
                    $performer= $performer_service->save($model);
                    // In case of any errors
                    if (!empty($performer)) {
                        $performer_ids[] = $performer->id;
                    }
                }
            }
            $event->performer=$performer_ids;
        }
        else{
            $event->performer= $performer;
        }
        $event->enable_performer= absint(event_m_get_param('enable_performer'));
        $event->match= absint(event_m_get_param('match'));
        $event_id = $this->dao->save($event);
        return $event_id;
    }
    
    public function save_social($event){
        $event->facebook_page=esc_url(event_m_get_param('facebook_page'));
        $event_id = $this->dao->save($event);
        return $event_id;
    }
    
    public function save_organizer($event){
        $event->organizer_name = sanitize_text_field(event_m_get_param('organizer_name'));
        $event->organizer_phones = event_m_get_param('organizer_phones');
        foreach($event->organizer_phones as $key => $val) { $event->organizer_phones[$key] = sanitize_text_field($val); }
        $event->organizer_emails = event_m_get_param('organizer_emails');
        foreach($event->organizer_emails as $key => $val) { $event->organizer_emails[$key] = sanitize_text_field($val); }
        $event->organizer_websites = event_m_get_param('organizer_websites');
        foreach($event->organizer_websites as $key => $val) { $event->organizer_websites[$key] = esc_url($val); }
        $event->hide_organizer = absint(event_m_get_param('hide_organizer'));
        $event_id = $this->dao->save($event);
        return $event_id;
    }
    
    public function save_venue($event){
        $venue = sanitize_text_field(event_m_get_param('venue'));
        $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
        $event->seating_capacity= absint(event_m_get_param('seating_capacity'));
        if ($venue=='new_venue') { // If admin chosen to add new Venue
            $model= new EventM_Venue_Model();
            $model->name= sanitize_text_field(event_m_get_param('new_venue'));;
            $model->type= 'standings';
            $model->address=sanitize_text_field(event_m_get_param('new_venue_address'));
            $model->seating_capacity=0;
            $venue = $venue_service->save($model);
            if (!empty($event->venue))
               $this->dao->remove_venue($event->id, $event->venue);
            $this->dao->set_venue($event->id, $venue->id);
            $event->venue=$venue->id;
        }else {
            // Save Venue info
            $venue = absint($venue);
            if (empty($venue)) {
                // Getting old venue
                if(!empty($event->venue)){
                    $this->dao->remove_venue($event->id,$event->venue);
                }
            }
            else {
                
                if($event->venue!=$venue){ // Venue changed.
                    $this->dao->set_venue($event->id, $venue);
                    $venue_model= $venue_service->load_model_from_db($venue);
                    if($venue_model->type!='standings'){
                        $event->seating_capacity=$venue_model->seating_capacity;
                        $event->seats=$venue_model->seats;
                    }
                    $event->venue=$venue;
                }
            }
        }
        $event_id = $this->dao->save($event);
        return $event_id;
    }
    
    public function save_user_submitted_event(){
        // Loading model
        $event = new EventM_Event_Model(0);
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs = $setting_service->load_model_from_db();
        
        $event->name = sanitize_text_field(event_m_get_param('name'));
        $event->description = wp_kses_post(stripslashes(event_m_get_param('description')));
        
        $event->event_type = absint(event_m_get_param('event_type'));
        
        // Event Dates
        $start_date = sanitize_text_field(event_m_get_param('start_date'));
        $event->start_date = !empty($start_date) ? em_timestamp($start_date) : '';
        $end_date = sanitize_text_field(event_m_get_param('end_date'));
        $event->end_date = !empty($end_date) ? em_timestamp($end_date) : '';
        $start_booking_date = sanitize_text_field(event_m_get_param('start_booking_date'));
        $event->start_booking_date = !empty($start_booking_date) ? em_timestamp($start_booking_date) : $event->start_date;
        $last_booking_date = sanitize_text_field(event_m_get_param('last_booking_date'));
        $event->last_booking_date = !empty($last_booking_date) ? em_timestamp($last_booking_date) : $event->end_date;
        
        $event->enable_booking = absint(event_m_get_param('enable_booking'));
        //$event->rm_form = absint(event_m_get_param('rm_form'));
        $event->all_day = absint(event_m_get_param('all_day'));
        //$event->audience_notice = wp_kses_post(stripslashes(event_m_get_param('audience_notice')));
        $event->status = $gs->ues_default_status;
        $event->ticket_price = (float) event_m_get_param('ticket_price');
        //$event->custom_link_enabled= absint(event_m_get_param('custom_link_enabled'));
        //$event->custom_link= event_m_get_param('custom_link');
        $event->user_submitted = absint(event_m_get_param('user_submitted'));
        
        $event->performer = event_m_get_param('performer');
        if(!empty($event->performer)){
            $event->enable_performer = 1;
        }
        // If admin chosen to add new Venue
        $new_venue = event_m_get_param('new_venue');
        $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
        if (!empty($new_venue)) {
            $address = event_m_get_param('new_venue_address');
            $seating_capacity = event_m_get_param('seating_capacity');
            $venue = $venue_service->map_request_to_model(0, array('name' => $new_venue, 'type' => 'standings', 'address' => $address, 'seating_capacity' => $seating_capacity));
            $venue_dao = new EventM_Venue_DAO();
            $venue = $venue_dao->save($venue);
        } else {
            $event->venue = absint(event_m_get_param('venue'));
        }
        // set feature image
        $cover_image_id = absint(event_m_get_param('attachment_id'));
        if (!empty($cover_image_id)) {
            $event->cover_image_id = $cover_image_id;
        }
        
        $event_id = $this->dao->save($event);
        
        // set vanue to event
        if(!empty($venue->id)){
            $this->dao->set_venue($event_id, $venue->id);
            $this->dao->set_meta($event_id, 'venue', $venue->id);
        }
        // cover image
        if (!empty($cover_image_id)) {
            $this->dao->set_thumbnail($event_id, $cover_image_id);
        }
        return $event_id;
    }
    
    public function save_popup_event($event){
        $old_event= clone $event;
        $event->all_day=absint(event_m_get_param('all_day'));
        $event->enable_booking=absint(event_m_get_param('enable_booking'));
        $start_date= sanitize_text_field(event_m_get_param('start_date'));
        $event->start_date= !empty($start_date) ? em_timestamp($start_date) : '';
        $end_date= sanitize_text_field(event_m_get_param('end_date'));
        $event->end_date= !empty($end_date) ? em_timestamp($end_date) : '';
        // Setting booking dates automatically if bookings are enabled
        if ($event->enable_booking === 1) {
            if (!isset($event->start_booking_date) || $event->start_booking_date == '') {
                $event->start_booking_date= em_get_local_timestamp();
                $event->last_booking_date= $event->start_date;
            } else {
                $event->start_booking_date= sanitize_text_field(event_m_get_param('start_booking_date'));
                $event->last_booking_date= sanitize_text_field(event_m_get_param('last_booking_date'));
            }
        } else {
            $event->start_booking_date= '';
            $event->last_booking_date= '';
        }
        $event->ticket_price=abs(floatval(event_m_get_param('ticket_price')));
        $event->event_type=absint(event_m_get_param('event_type'));
        $venue = absint(event_m_get_param('venue'));
        // Save Venue info
        if (empty($venue)) {
            // Getting old venue
            if(!empty($event->venue)){
                $this->dao->remove_venue($event->id,$event->venue);
            }
        }
        else {
            if($event->venue!=$venue){ // Venue changed.
                $this->dao->set_venue($event->id, $venue);
                $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
                $venue_model= $venue_service->load_model_from_db($venue);
                if($venue_model->type!='standings' && (empty($event->seating_capacity) || $event->seating_capacity>$venue_model->seating_capacity)){
                    $event->seating_capacity=$venue_model->seating_capacity;
                    $event->seats=$venue_model->seats;
                }
            }
        }
        $event->venue= $venue;
        $event->performer= event_m_get_param('performer');     
        $event->name= htmlspecialchars_decode(sanitize_text_field(strip_tags(event_m_get_param('title'))));
        $event->enable_performer= absint(event_m_get_param('performer'));
        $event->status= event_m_get_param('status');
        $event = apply_filters('ep_before_saving_popup_event',$event);
        $event_id = $this->dao->save($event);
        do_action('ep_popup_event_saved',$old_event,$event_id);
        return $event_id;
    }
   
    public function save($model) {
        $id = isset($model->id) ? $model->id : 0;
        $model = $this->map_request_to_model($id, $model);
        // If admin chosen to add new Performer(s)
        $performer_dao = new EventM_Performer_DAO();
        $custom_performer_name = event_m_get_param('custom_performer_name');
        if (!empty($custom_performer_name)) {
            // Check if multiple performers given seperated by comma
            $performers = explode(',', $custom_performer_name);
            $performer_ids = array();
            foreach ($performers as $performer_name){
                if (!empty($performer_name)){
                    $performer = new EventM_Performer_Model();
                    $performer->name = $performer_name;
                    $type = event_m_get_param('custom_performer_type');
                    $performer->type= empty($type) ? 'person': $type;
                    $slug = event_m_get_param('custom_performer_slug');
                    $performer->slug= empty($slug) ? '': $slug;
                    $performer= $performer_dao->save($performer);
                    // In case of any errors
                    if (!empty($performer)) {
                        $performer_ids[] = $performer->id;
                    }
                }
            }
            $model->performer=$performer_ids;
        }
        //Booking dates
        $model->start_booking_date= empty($model->start_booking_date) ? $model->start_date : $model->start_booking_date;
        $model->last_booking_date= empty($model->last_booking_date) ? $model->end_date : $model->last_booking_date;
        
        $event_id = $this->dao->save($model);
        // In case of any errors
        if ($event_id instanceof WP_Error) {
            return $event_id;
        }
        $event = $this->load_model_from_db($event_id);
        
        // If admin chosen to add new Event Type
        $type_dao = new EventM_Event_Type_DAO();
        $new_event_type = event_m_get_param('new_event_type');
        $new_event_type_color = event_m_get_param('new_event_type_color');
        if (!empty($new_event_type)) {
            $event_type_service = EventM_Factory::get_service('EventTypeM_Service');
            $event_type = $event_type_service->map_request_to_model(0, array('name' => $new_event_type, 'color' => $new_event_type_color));
            $type = $type_dao->save($event_type);
            $this->dao->set_type($event->id, $type->id);
            $this->dao->set_meta($event->id, 'event_type', $type->id);
        } else {
            $type = $model->event_type;
            if ($type>0){
                $this->dao->set_type($event->id, $type);
            }
                
        }

        // If admin chosen to add new Venue
        $new_venue = event_m_get_param('new_venue');
        $venue_service = EventM_Factory::get_service('EventM_Venue_Service');
        if (!empty($new_venue)) {
            $address = event_m_get_param('new_venue_address');
            $seating_capacity = event_m_get_param('new_venue_capacity');
            $venue = $venue_service->map_request_to_model(0, array('name' => $new_venue, 'type' => 'standings', 'address' => $address, 'seating_capacity' => $seating_capacity));
            $venue_dao = new EventM_Venue_DAO();
            $venue = $venue_dao->save($venue);
            if (!empty($model->venue))
               $this->dao->remove_venue($event->id, $model->venue);
            $this->dao->set_venue($event->id, $venue->id);
            $this->dao->set_meta($event->id, 'venue', $venue->id);
        } else {
            // Save Venue info
            $venue = absint(event_m_get_param('venue'));
            if (empty($venue)) {
                    // Getting old venue
                    $old_venue= $this->dao->get_venue($event->id);
                    if(!empty($old_venue)){
                        $this->dao->remove_venue($event->id,$old_venue->term_id);
                    }
            }
            else {
                $this->dao->set_venue($event->id, $venue);
                $venue_model= $venue_service->load_model_from_db($venue);
                if($venue_model->type!='standings' && $event->seating_capacity>$venue_model->seating_capacity){
                    $event->seating_capacity=$venue_model->seating_capacity;
                    $event->seats=$venue_model->seats;
                }
                
                $this->update_model($event);
            }
        }

        // Set Feature image
        $cover_image_id = $event->cover_image_id;
        if ($cover_image_id != null && (int) $cover_image_id > 0) {
            $this->dao->set_thumbnail($event->id, $cover_image_id);
        }

        // If cover image not uploaded then set first image from gallery
        if ($cover_image_id == null || (int) $cover_image_id == 0) {
            if (is_array($event->gallery_image_ids) && !empty($event->gallery_image_ids)) {
                $this->dao->set_thumbnail($event->id, $gallery_image_ids[0]);
            }
        }

        do_action('event_magic_event_saved', $event);
        return $event;
    }
    
    private function update_meta_before_save($meta, $model) {

        if (is_array($meta)) {
            foreach ($meta as $value) {
                if ($value == "allow_discount" && 0 == (int) $model->$value) {
                    $index = array_search('discount_no_tickets', $meta);
                    unset($meta[$index]);

                    $index = array_search('discount_per', $meta);
                    unset($meta[$index]);
                }
            }
        }

        return $meta;
    }

    public function get_venue_capcity($venue_id, $event_id) {
        $response = array();
        $service = EventM_Factory::get_service('EventM_Venue_Service');
        $response['capacity'] = (int) $service->capacity($venue_id);
        $response['seats'] = $service->get_seats($venue_id, $event_id);

        return $response;
    }

    public function validate($model) {
        $errors= array();
        if ($model->venue == 'new_venue')
        {
            $new_venue= event_m_get_param('new_venue',true);
            if(term_exists($new_venue, EM_EVENT_VENUE_TAX)){
                $errors[]= __('Please use different Venue name','eventprime-event-calendar-management');
            }
        }
        
        if ($model->event_type == 'new_event_type')
        {
            $new_event_type= event_m_get_param('new_event_type',true);
            if(term_exists($new_event_type, EM_EVENT_TYPE_TAX)){
                $errors[]= __('Please use different Event Type','eventprime-event-calendar-management');
            }
 
        }
        
        if(!empty($model->enable_booking)){
            
            $start_date= em_timestamp($model->start_date);
            $end_date= em_timestamp($model->end_date);
            $last_booking_date = em_timestamp($model->last_booking_date);
            $start_booking_date =  em_timestamp($model->start_booking_date);
            
            if ($start_date > $end_date) {
            $errors[]= __('Event Start date should be prior to Event End date.','eventprime-event-calendar-management');
            }

            if ($last_booking_date> $end_date) {
                $errors[]=__('Last booking date can not be greater than End date.','eventprime-event-calendar-management');
            }

            if ($start_booking_date > $last_booking_date) {
                $errors[]=__('Start booking date should be earlier than the Last Booking date.','eventprime-event-calendar-management');
            }
            if ($start_booking_date > $start_date) {
                $errors[]=__('Start booking date must be earlier than Event Start date.','eventprime-event-calendar-management');
            }
            if ($start_booking_date == $last_booking_date) {
                $errors[]=__('Start booking date must be earlier than end booking date.','eventprime-event-calendar-management');
            }
        }
        $errors= apply_filters('event_magic_event_save_validation',$errors,$model);
        return $errors;
    }

    public function get_events_the_query($custom_args=array(),$shortcode_params=array()) {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs = $setting_service->load_model_from_db();
        $hide_past_events = $gs->hide_past_events;
        $type_ids = $shortcode_params['types'];
        $venue_ids = $shortcode_params['sites'];
        $viewtype = $shortcode_params['view'];
        $posts_per_page = 10;
        if($viewtype == 'card' || (isset($_GET['events_view']) && $_GET['events_view'] == 'card')){
            $show_no_of_events_card = em_global_settings('show_no_of_events_card');
            $posts_per_page = $show_no_of_events_card;
            if($show_no_of_events_card == 'custom'){
                $posts_per_page = em_global_settings('card_view_custom_value');
            }
            if($show_no_of_events_card == 'all'){
                $posts_per_page = -1;
            }
            if(isset($shortcode_params['show']) && !empty($shortcode_params['show'])){
                $posts_per_page = $shortcode_params['show'];
            }
        }
        $paged = (get_query_var('paged') ) ? get_query_var('paged') : 1;

        $meta_query= array(
            'relation' => 'AND',
            array(
                'key' => em_append_meta_key('hide_event_from_events'),
                'value' => '1',
                'compare' => '!='
            )
        );
        $search = event_m_get_param('em_s');
        $args = array(
            'meta_key' => em_append_meta_key('start_date'),
            'orderby' => 'meta_value',
            'order' => 'ASC',
            'posts_per_page' => $posts_per_page,
            'paged' => $paged,
            'meta_query' => $meta_query,
            'post_type' => EM_EVENT_POST_TYPE
        );
        $args['post_status'] = !empty($hide_past_events) == 1 ? 'publish' : 'any';
        
        if ($search == "1"){
            $sd = event_m_get_param('em_sd');
            $start_date= DateTime::createFromFormat('!Y-m-d',$sd);
            if(!empty($start_date)){
                $start_date->setTime(23,59,59);
                $start_ts= $start_date->getTimestamp();
                $start_date->setTime(0,0);
                $end_ts=$start_date->getTimestamp();
                array_push($meta_query,array(
                    'key' => em_append_meta_key('start_date'),
                    'value' => $start_ts,
                    'compare' => '<=',
                    'type'=>'NUMERIC'
                ),
                array(
                    'key' => em_append_meta_key('end_date'),
                    'value' => $end_ts,
                    'compare' => '>=',
                    'type'=>'NUMERIC'
                ));
            }
            
            $keyword = event_m_get_param('em_search');
            if (!empty($keyword)){
                $args['s']= $keyword;
            }
            
            $types = event_m_get_param('em_types');
            if (!empty($types)) {
                $types = is_array($types) ? $types : array($types);
                if(!empty($type_ids))
                    $type_ids = array_intersect($types,$type_ids);
                else
                    $type_ids = $types;
            }
            
            $venue = event_m_get_param('em_venue');
            if (!empty($venue)) {
                $venue = is_array($venue) ? $venue : array($venue);
                if(!empty($venue_ids))
                    $venue_ids = array_intersect($venue,$venue_ids);
                else
                    $venue_ids = $venue;
            }
        }
        
        if (!empty($type_ids)) {
            array_push($meta_query,array(
                'key' => em_append_meta_key('event_type'),
                'value' => $type_ids,
                'compare' => 'IN',
                'type'=>'NUMERIC'
            ));
        }
        
        if (!empty($venue_ids)) {
            array_push($meta_query,array(
                'key' => em_append_meta_key('venue'),
                'value' => $venue_ids,
                'compare' => 'IN',
                'type'=>'NUMERIC'
            ));
        }
        
        if(!empty($meta_query)){
            $args['meta_query']= $meta_query;
        }
        
        $args = wp_parse_args($custom_args, $args);
        // if card view then shows only publish posts
        if(($viewtype == 'card' || (isset($_GET['events_view']) && $_GET['events_view'] == 'card')) && $posts_per_page != -1){
            $args['post_status'] = 'publish';
        }
        // post filter
        add_filter('posts_orderby', 'em_posts_order_by');

        // check for future entries
        $sort_type = isset($_REQUEST['sort_type']) ? sanitize_text_field($_REQUEST['sort_type']) : '';
        if(!empty($sort_type) && $sort_type == 'newer'){
            $args['post_status'] = 'publish';
        }
        // check for past entries
        if(!empty($sort_type) && $sort_type == 'older'){
            $args['post_status'] = 'expired';
            $args['paged'] = $paged - 1;
            $args['order'] = 'DESC';
            add_filter('posts_orderby', 'em_older_posts_order_by');
        }

        $wp_query = new WP_Query($args);
        remove_filter('posts_orderby', 'em_posts_order_by');

        if(!empty($sort_type) && $sort_type == 'older'){
            remove_filter('posts_orderby', 'em_older_posts_order_by');
        }

        // check for newer entries exists
        $newer_args = $args;
        $newer_args['paged'] = $paged + 1;
        $newer_args['post_status'] = 'publish';
        add_filter('posts_orderby', 'em_posts_order_by');
        $next_page_query = new WP_Query($newer_args);
        remove_filter('posts_orderby', 'em_posts_order_by');
        if(isset($next_page_query->posts) && count($next_page_query->posts) > 0 && $posts_per_page != -1){
            $wp_query->next_page_exist = 1;
        }

        // check for older entries exists
        $older_args = $args;
        $older_args['paged'] = $paged;
        $older_args['post_status'] = 'expired';
        add_filter('posts_orderby', 'em_older_posts_order_by');
        $previous_page_query = new WP_Query($older_args);
        remove_filter('posts_orderby', 'em_older_posts_order_by');
        if(isset($previous_page_query->posts) && count($previous_page_query->posts) > 0 && $posts_per_page != -1 && empty($hide_past_events)){
            $wp_query->previous_page_exist = 1;
        }
        if(!empty($sort_type) && $sort_type == 'older' && $posts_per_page != -1){
            if($paged > 1){
                $wp_query->older_page_exist = 1;       
            }
        }

        return $wp_query;
    }

    public function get_upcoming_events($exclude = array()) {
        $events = $this->dao->get_upcoming_events();
        if (empty($events))
            return array();
        $temp = array();
        foreach ($events as $event) {
            $model = $this->load_model_from_db($event->ID);
            array_push($temp, $model);
        }
        return $temp;
    }

    private function get_upcoming_date($dates = array()) {
        $upcoming_date = null;

        if (!empty($dates)):
            foreach ($dates as $date):
                if (strtotime($date) >= strtotime(date('Y-m-d'))):
                    $upcoming_date = $date;
                    break;
                endif;
            endforeach;
        endif;

        return $upcoming_date;
    }

    public function get_data_for_slider() {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $global_settings= $setting_service->load_model_from_db();
        $data = new stdClass();
        $data->image_ids = array();
        $data->links = array();
        $data->ids = array();
        $args = array(
            'numberposts' => -1,
            'order' => 'ASC',
            'post_status' => 'publish',
            'meta_query' => array('relation' => 'AND', // WordPress has all the results, now, return only the events after today's date
                array(
                    'relation' => 'AND',
                    array(
                        'key' => em_append_meta_key('hide_event_from_events'),
                        'value' => '1', //
                        'compare' => '!='
                    ),
                    array(
                        'relation' => 'OR',
                        array(
                            'key' => em_append_meta_key('start_date'), 
                            'value' => current_time('timestamp'), 
                            'compare' => '>=', 
                        ),
                        array(
                            'key' => em_append_meta_key('end_date'),
                            'value' => current_time('timestamp'),
                            'compare' => '>=', 
                        )))
            ),
            'post_type' => EM_EVENT_POST_TYPE);

        $events = $this->dao->get_events($args);
        foreach ($events as $event){
            $event = $this->load_model_from_db($event->ID);
            $image_id = get_post_thumbnail_id($event->id);
            if (!empty($image_id)) {
                $data->image_ids[] = $image_id;
            } else {
                $data->image_ids[] = '';
            }
            $data->links[] = (absint($event->custom_link_enabled) == 1) ? $event->custom_link : add_query_arg('event', $event->id, get_page_link($global_settings->events_page));

            $data->ids[] = $event->id;
        }     
        return $data;
    }

    public function searchByText($text, $exclude = array()) {
        $search_query = new WP_Query();
        $events = $search_query->query('s=' . $text);
        $event_ids = array();
        // echo'<pre>'; var_dump($events); 

        foreach ($events as $key => $event):
            if (!in_array($event->ID, $exclude)):
                $event_ids[] = $event->ID;
            endif;
        endforeach;

        return $event_ids;
    }

    public function searchByType($type, $exclude = array()) {
        $event_ids = array();
        $event_service= EventM_Factory::get_service('EventM_Service');
        $events = $event_service->events_by_type($type);

        foreach ($events as $key => $event){
            if (!in_array($event->ID, $exclude)){
                $event_ids[] = $event->ID;
            }
        }


        return ($event_ids);
    }


    public function update_past_event_status() {
        $event_dao = new EventM_Event_DAO();
        $events = $event_dao->get_past_events();

        if (!empty($events)):
            foreach ($events as $event):
                wp_update_post(array('ID' => $event->ID, 'post_status' => 'expired'));
            endforeach;
        endif;
    }

    public function get_all_events() {
        $event_dao = new EventM_Event_DAO();
        return $event_dao->get_events();
    }

    public function get_post($ID) {
        $args = array(
            'author' => $ID,
            'orderby' => 'post_date',
            'order' => 'ASC',
            'post_type' => EM_EVENT_POST_TYPE
        );
        $details = get_posts($args);
        return $details;
    }

    public function get_venue($event_id) {
        $venue = $this->dao->get_venue($event_id);
        if (!empty($venue)) {
            return $venue->term_id;
        }
        return null;
    }

    public function get_performer($event_id) {
        $performer = em_get_post_meta($event_id, 'performer', true);
        return $performer;
    }

    public function get_type($event_id) {
        $type = $this->dao->get_type($event_id);
        if (!empty($type))
            return $type->term_id;
        return null;
    }

    public function get_events($filter) {
        return $this->dao->get_events($filter);
    }

    public function available_seats($event_id) {
        return $this->dao->available_seats($event_id);
    }

    public function booked_seats($event_id) {
        return (int) $this->dao->booked_seats($event_id);
    }

    public function get_header($event) {
        $local_objects = array();
        $setting_service= EventM_Factory::get_service('EventM_Setting_Service');
        $options= $setting_service->load_model_from_db();
        wp_enqueue_script('jquery-ui-tabs', array('jquery'));
        //get_header();
        wp_enqueue_script('em-single-event');
        wp_enqueue_script('jquery-colorbox');
        if (!empty($options->gcal_sharing)){
            wp_enqueue_script("em-gcal", plugin_dir_url(__DIR__) . 'templates/js/em-gcal.js', array(), EVENTPRIME_VERSION);
            wp_enqueue_script("em-google-client", "https://apis.google.com/js/client.js?onload=em_gcal_handle", array(), EVENTPRIME_VERSION);
            wp_localize_script("em-gcal", "em_local_gcal_objects", array("gc_id" => $options->google_cal_client_id, "g_api_key" => $options->google_cal_api_key));
        }

        if ($options->social_sharing && !empty($options->fb_api_key)){
            $local_objects["social_sharing"] = 1;
            $local_objects["fb_api"] = $options->fb_api_key;
        }
        
        $event_url=add_query_arg(array('event'=>$event->id),get_permalink($options->events_page));
        $local_objects["fb_event_href"] =$event_url;
        wp_localize_script("em-single-event", "em_local_event_objects", $local_objects);
        em_localize_map_info("em-google-map");
    }

    public function load_model_from_db($id) {
        $event= $this->dao->get($id);
        $event= $this->format_model_from_db($event);
        return apply_filters('loading_event_from_db',$event);
    }

    public function get_venues_dropdown() {
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $dropdown = array();
        $venues = $venue_service->get_venues();
        // Insert default value 
        $tmp = new stdClass();
        $tmp->id = 0;
        $tmp->name = __('None', 'eventprime-event-calendar-management');
        $dropdown[] = $tmp;

        foreach ($venues as $venue) {
            $tmp = new stdClass();
            $tmp->id = $venue->id;
            $tmp->name = $venue->name;
            $tmp->seating_capacity = $venue->seating_capacity;
            $dropdown[] = $tmp;
        }

        $tmp = new stdClass();
        $tmp->name = __('Add New Site', 'eventprime-event-calendar-management');
        $tmp->id = "new_venue";
        $dropdown[] = $tmp;

        return $dropdown;
    }

    public function get_performers_dropdown() {
        $performer_dao = new EventM_Performer_DAO();
        $dropdown = array();
        $performers = $performer_dao->get_all();

        $tmp = new stdClass();
        $tmp->id = 0;
        $tmp->name = __("Select", 'eventprime-event-calendar-management');
        //$dropdown[] = $tmp;

        if ($performers != null) {
            foreach ($performers as $performer) {
                $tmp = new stdClass();
                $tmp->name = $performer->name;
                $tmp->id = $performer->id;
                $dropdown[] = $tmp;
            }
        }

        $tmp = new stdClass();
        $tmp->name = __('New Performer', 'eventprime-event-calendar-management');
        $tmp->id = "new_performer";
        $dropdown[] = $tmp;

        return $dropdown;
    }

    public function get_types_dropdown() {
        $type_dao = new EventM_Event_Type_DAO();
        $event_types = array();
        $types = $type_dao->get_all();

        // Insert default value 
        $tmp = new stdClass();
        $tmp->id = 0;
        $tmp->name = __('Select', 'eventprime-event-calendar-management');
        $event_types[] = $tmp;

        if ($types != null) {
            foreach ($types as $type) {
                $tmp = new stdClass();
                $tmp->name = $type->name;
                $tmp->id = $type->id;
                $event_types[] = $tmp;
            }
        }

        $tmp = new stdClass();
        $tmp->name = __('Add Event Type', 'eventprime-event-calendar-management');
        $tmp->id = "new_event_type";
        $event_types[] = $tmp;

        return $event_types;
    }

    public function get_ticket_dropdown() {
        $dropdown = array();
        $templates = apply_filters('event_magic_ticket_templates', array());

        $tmp = new stdClass();
        $tmp->id = 0;
        $tmp->name = __('Select', 'eventprime-event-calendar-management');
        $dropdown[] = $tmp;

        if ($templates != null) {
            foreach ($templates as $template) {
                $tmp = new stdClass();
                $tmp->name = $template->post_title;
                $tmp->id = $template->ID;
                $dropdown[] = $tmp;
            }
        }

        return $dropdown;
    }

    public function map_request_to_model($id, $model = null) {
        $event = new EventM_Event_Model($id);
        $data = (array) $model;

        if (!empty($data) && is_array($data)) {
            foreach ($data as $key => $val) {
                if(property_exists($event,$key)){
                    $event->{$key}= $val;
                }
            }
        }
        return $this->format_model_to_save($event);
    }
    
    private function format_model_to_save($model){
        $model->allow_cancellations= absint($model->allow_cancellations);
        $model->allow_discount= absint($model->allow_discount);
        $model->booked_seats= absint($model->booked_seats);
        $model->cover_image_id= absint($model->cover_image_id);
        $model->discount_no_tickets= absint($model->discount_no_tickets);
        $model->discount_per= (float) $model->discount_per;
        $model->enable_booking= absint($model->enable_booking);
        $model->enable_performer= absint($model->enable_performer);
        $model->start_date= !empty($model->start_date) ? em_timestamp($model->start_date) : '';
        $model->end_date= !empty($model->end_date) ? em_timestamp($model->end_date) : '';
        $model->start_booking_date= !empty($model->start_booking_date) ? em_timestamp($model->start_booking_date) : '';
        $model->last_booking_date= !empty($model->last_booking_date) ? em_timestamp($model->last_booking_date) : '';
        $model->event_type= absint($model->event_type);
        $model->hide_booking_status= absint($model->hide_booking_status);
        $model->hide_event_from_calendar= absint($model->hide_event_from_calendar);
        $model->hide_event_from_events= absint($model->hide_event_from_events);
        $model->match= absint($model->match);
        $model->max_tickets_per_person= absint($model->max_tickets_per_person);
        $model->ticket_template= absint($model->ticket_template);
        $model->venue= absint($model->venue);
        $model->rm_form= absint($model->rm_form);
        $model->en_ticket= absint($model->en_ticket);
        $model->seating_capacity= absint($model->seating_capacity);
        $model->ticket_price= (float) $model->ticket_price;
        $model->performer= is_array($model->performer) ? $model->performer : array();
        $model->description= wp_kses_post(stripslashes($model->description));
        $model->hide_organizer= absint($model->hide_organizer);
        $model->enable_attendees= absint($model->enable_attendees);
        $model->show_attendees= absint($model->show_attendees);
        $model= apply_filters('event_magic_format_model_to_save',$model);
        return $model;
    }
    
    private function format_model_from_db($model){
        $model->allow_cancellations= absint($model->allow_cancellations);
        $model->allow_discount= absint($model->allow_discount);
        $model->booked_seats= $this->booked_seats($model->id);
        $model->cover_image_id= absint($model->cover_image_id);
        $model->discount_no_tickets= absint($model->discount_no_tickets);
        $model->discount_per= (float) $model->discount_per;
        $model->enable_booking= absint($model->enable_booking);
        $model->enable_performer= absint($model->enable_performer);
        $model->event_type= absint($model->event_type);
        $model->hide_booking_status= absint($model->hide_booking_status);
        $model->hide_event_from_calendar= absint($model->hide_event_from_calendar);
        $model->hide_event_from_events= absint($model->hide_event_from_events);
        $model->hide_organizer= absint($model->hide_organizer);
        $model->match= absint($model->match);
        $model->max_tickets_per_person= absint($model->max_tickets_per_person);
        $model->ticket_template= absint($model->ticket_template);
        $model->venue= absint($model->venue);
        $model->rm_form= absint($model->rm_form);
        $model->en_ticket= absint($model->en_ticket);
        $model->seating_capacity= absint($model->seating_capacity);
        $model->ticket_price= (float) $model->ticket_price;
        $model->all_day= absint($model->all_day);
        $model->custom_link_enabled= absint($model->custom_link_enabled);
        $model->user_submitted= absint($model->user_submitted);
        $model->user= absint($model->user);
        $model->enable_attendees= absint($model->enable_attendees);
        $model->show_attendees= absint($model->show_attendees);
        $model->performer= !is_array($model->performer) ? array() : $model->performer;
        // Checking for Seating extension for ticket prices
        $em= event_magic_instance(); 
        if(!in_array('seating',$em->extensions) || empty($model->en_ticket)){
            $model->max_tickets_per_person=0;
        }
        $model= apply_filters('event_magic_format_model_from_db',$model);
        return $model;
    }
    
    public function update_booked_seats($event, $no_seats) {
        // Get parent event
        $event_status = get_post_status($event);
        if ($event_status == 'publish') {
            $parent = $this->dao->get_meta($event, 'parent_event');
            if (!empty($parent)) {
                //$prev_parent_booked_seats = (int) $this->dao->get_meta($parent, 'booked_seats');
                $prev_parent_booked_seats = (int) $this->dao->booked_seats($parent);
                //$prev_child_booked_seats = (int) $this->dao->get_meta($event, 'booked_seats');
                $prev_child_booked_seats = (int) $this->dao->booked_seats($event);
                if ($prev_child_booked_seats > $no_seats) {
                    $diff_seats = $prev_child_booked_seats - $no_seats;
                    $this->dao->set_meta($parent, 'booked_seats', $prev_parent_booked_seats - $diff_seats);
                } else if ($no_seats > $prev_child_booked_seats) {
                    $diff_seats = $no_seats - $prev_child_booked_seats;
                    $this->dao->set_meta($parent, 'booked_seats', $prev_parent_booked_seats + $diff_seats);
                }
            }
            $this->dao->set_meta($event, 'booked_seats', $no_seats);
        }
    }

    /*
     * Event Settings Page 
     */

    public function load_settings_page() {
        $id = event_m_get_param('post_id');
        $event = $this->load_model_from_db($id);
        if (empty($event)) {
            $error = new WP_Error('NON_EXIST_EVENT', "Event does not exists");
            echo wp_json_encode($error);
            wp_die();
        }
        $event->event_types = $this->get_types_dropdown();
        $event->cover_image_id = get_post_thumbnail_id($id);
        $event->cover_image_url= $this->get_event_cover_image($id);
        $event->images = $this->get_event_images($response->post['gallery_image_ids']);
        $event->rm_forms = $this->get_rm_forms();
        $event->status_list = em_array_to_options(array(
            "publish" =>__('Active','eventprime-event-calendar-management'),
            "expired" => __('Unpublished','eventprime-event-calendar-management'),
            "draft" =>__('Draft','eventprime-event-calendar-management')
        ));
        return $event;
    }

    public function get_events_for_calendar_view($params) {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $global_settings= $setting_service->load_model_from_db();
        $currency_symbol = "";
        $em = event_magic_instance();
        $date_format = get_option('date_format');
        $time_format = get_option('time_format');
        $date_time_format = $date_format . ' ' . $time_format;
        if ($global_settings->currency) {
            $all_currency_symbols = EventM_Constants::get_currency_symbol();
            $currency_symbol = $all_currency_symbols[$global_settings->currency];
        } else {
            $currency_symbol = EM_DEFAULT_CURRENCY;
        }
        $the_query = $this->get_events_the_query(array('nopaging'=>true),$params);
        $posts = $the_query->posts;
        $events = array();
        $performer_service = EventM_Factory::get_service('EventM_Performer_Service');
        $type_service = EventM_Factory::get_service('EventTypeM_Service');
        if (empty($posts))
            return $events;
        
        $posts = apply_filters('ep_filter_front_events',$posts,$params);
        $posts = array_filter($posts, function($post){ return $post->post_status !== 'draft'; });
        foreach ($posts as $post) {
            $event = $this->load_model_from_db($post->ID);
            $time_str = '';
            $start_date = em_showDateTime($event->start_date,true,"m/d/Y");
            $end_date = em_showDateTime($event->end_date,true,"m/d/Y");
            if (em_compare_event_dates($post->ID)) {
                //$day = date('D, M d, Y',$event->start_date);
                $day = date_i18n($date_format,$event->start_date);
                $start_time = date_i18n($time_format,$event->start_date);
                $end_time = date_i18n($time_format,$event->end_date);                
                $time_str .= $day . ' ' . $start_time . ' to ' . $end_time;
            } 
            else {
                $time_str .= date_i18n($date_time_format,$event->start_date) . ' - ' . date_i18n($date_time_format,$event->end_date);
            }
            $link_href = add_query_arg("event", $event->id, get_permalink($global_settings->events_page));
            $event_arr = array('title' => $event->name, 'start' => $start_date, 'end' => $end_date, 'id' => $post->ID,
                'time_str' => $time_str);
            if (!empty($event->event_type)) {
                $type_model = $type_service->load_model_from_db($event->event_type);
                $event_arr['bg_color'] = '#' . $type_model->color;
            }
            if(!empty($event->event_text_color)){
                $event_arr['event_text_color'] = '#' . $event->event_text_color;
            }
            $venue_address = em_get_term_meta($event->venue, 'address', true);
            $event_arr['address']= !empty($venue_address) ? $venue_address : '';
            $event_arr['ticket_price']= (abs(floatval($event->ticket_price))<=0) ? '' : em_currency_symbol().$event->ticket_price;
            $bookable= $this->is_bookable($event);
            if($bookable){
                $showBookNowForGuestUsers = em_show_book_now_for_guest_users();
                $current_ts = em_current_time_by_timezone();
                if($event->status=='expired'){
                    $event_arr['link']='<div class="em_header_button em_event_expired kf-tickets">'.__("Bookings Expired",'eventprime-event-calendar-management').'</div>';
                }
                elseif($current_ts>$event->last_booking_date){
                    $event_arr['link']='<div class="em_header_button em_booking-closed kf-tickets">'.__("Bookings Closed",'eventprime-event-calendar-management').'</div>';
                }
                elseif($current_ts<$event->start_booking_date){
                    $event_arr['link']='<div class="em_header_button em_not_started kf-tickets">'.__("Bookings not started yet",'eventprime-event-calendar-management').'</div>';
                }
                else {
                    if (is_user_logged_in() || $showBookNowForGuestUsers) {
                        $event_arr['link'] = '<form action="'.get_permalink($global_settings->booking_page).'" method="post" name="em_booking"><input type="hidden" name="event_id" value="'.$event->id.'" /><input type="hidden" name="venue_id" value="'.$event->venue.'" /><button name="tickets" onclick="em_event_booking('.$event->id.')" class="em_color">'.__('Book Now','eventprime-event-calendar-management').'</button>';
                    }
                    else {
                        $event_arr['link'] = '<a class="em_header_button kf-tickets" target="_blank" href="'.add_query_arg('event_id',$event->id,get_permalink($global_settings->profile_page)).'">'.__('Book Now','eventprime-event-calendar-management').'</a>';
                    }
                }
            }
            else {
                if((isset($event->parent) && !empty($event->parent)) && (isset($event->enable_recurrence_automatic_booking) && !empty($event->enable_recurrence_automatic_booking))){
                    $event_arr['link']='<div class="em_header_button em_booking-closed kf-tickets">'.__("Bookings not allowed",'eventprime-event-calendar-management').'</div>';
                }
                else{
                    $event_arr['link']='<div class="em_header_button em_booking-closed kf-tickets">'.__("Bookings Closed",'eventprime-event-calendar-management').'</div>';
                }
            }
            
            $capacity = em_event_seating_capcity($event->id);
            if(!empty($event->enable_booking) && empty($event->hide_booking_status) && !empty($capacity)){
               $sum = $this->booked_seats($event->id);
               $width= ($sum / $capacity) * 100;
               $event_arr['booking_status']= array('stats'=>$sum.' / '.$capacity,'width'=>$width);
            }
            if (!empty($event->enable_performer) && !empty($event->performer)) {
                $performers = array();
                foreach ($event->performer as $p_id) {
                    $performer = $performer_service->load_model_from_db($p_id);
                    $cover_image = $performer_service->get_image($performer->id);
                    if (!empty($cover_image)) {
                        array_push($performers, $cover_image);
                    }
                }
                if (!empty($performers)) {
                    $event_arr['performers'] = '<ul>';
                    foreach ($performers as $img_url) {
                        $event_arr['performers'] .= "<li><img src='$img_url' /></li>";
                    }
                    $event_arr['performers'] .= '</ul>';
                }
            }
            $event_arr['url']= (absint($event->custom_link_enabled) == 1) ? $event->custom_link : add_query_arg('event', $post->ID, get_page_link($global_settings->events_page));
            $event_arr['ticket_price'] = apply_filters('event_magic_load_calender_ticket_price', $event_arr['ticket_price'], $event);
            $popup_html = '<div class="em_event_detail_popup" style="display:none">
                            <a href="'.$event_arr['url'].'">
                            <div class="ep_event_detail_popup_head em_bg dbfl">
                                <div class="ep_event_title difl">'.$event_arr['title'].'</div>
                                <div class="ep_event_price difr">'.$event_arr['ticket_price'].'</div>
                            </div>
                            </a>
                            <div class="ep-event-detail-wrap">
                                <div class="ep-event-detail-row dbfl"><i class="material-icons">access_time</i></span><span class="ep_event_time">'.$event_arr['time_str'].'</span></div>';
            if(!empty($event_arr['address'])){
                $popup_html .= '<div class="ep-event-location ep-event-detail-row dbfl"><i class="material-icons">location_on</i><span class="ep_event_venue">'.$event_arr['address'].'</span></div>';
            }
            
            if(!empty($event_arr['performers'])){
                $popup_html .= '<div class="ep-event-performers ep-event-detail-row dbfl"><span class="ep-event-performers-title">'.__('Performers','eventprime-event-calendar-management').'</span><div class="ep_event_performers">'.$event_arr['performers'].'</div></div>';
            }
            
            if(!empty($event_arr['booking_status']) && absint($event->custom_link_enabled) != 1){
                $popup_html .= '<div class="event_booking_details ep-event-detail-row">'.
                                    __("Booking Status", 'eventprime-event-calendar-management').
                                '<div class="status">'.$event_arr['booking_status']['stats'].'</div>
                                 <div class="dbfl">
                                    <div id="progressbar" class="em_progressbar dbfl">
                                        <div class="em_progressbar_fill em_bg" style="width:'.$event_arr['booking_status']['width'].'%"></div>
                                    </div>
                                </div>'; 
            }
            if(!empty($event->enable_booking) && absint($event->custom_link_enabled) != 1){
                $popup_html .= '<div class="ep-booking-footer"><div class="ep-book-now dbfl">'.$event_arr['link'].'</div>';
            }
            // code for wishlist extension
            if(in_array('wishlist',$em->extensions)){
                if(is_user_logged_in()):
                    ob_start();
                    $event->has_popup = 1;
                    do_action('event_magic_wishlist_link',$event);
                    $wishlist = ob_get_contents();
                    ob_end_clean();
                    $popup_html .= $wishlist;
                endif;
            }
            // for custom data on calendar from extensions
            ob_start();
            do_action('event_magic_popup_custom_data',$event);
            $popup_custom_data = ob_get_contents();
            ob_end_clean();
            $popup_html .= $popup_custom_data;
            $popup_html .= '</div></div>';
            $event_arr['popup_html']=  $popup_html;
            array_push($events, $event_arr);
        }
        return $events;
    }
    
    public function create_seats_from_venue($event_id, $venue_id) {
        $data = new stdClass();
        $event_seats = array();
        $venue_seats = array();
        $event_seats = em_get_post_meta($event_id, 'seats', true);
        $venue_seats = em_get_term_meta($venue_id, 'seats', true);
     
        $data->event_id = $event_id;
        if (empty($venue_seats)) { 
            $data->seats = array();
            em_update_post_meta($event_id, 'seats', array());
            return $data;
        }
        // Copy seat structure in case it not copied already
        if (empty($event_seats)) {  
            em_update_post_meta($event_id, 'seats', $venue_seats);
          
            $data->seats = $venue_seats;
            return $data;
        } else {
            $event_seats = em_get_post_meta($event_id, 'seats', true);
            $data->seats = $event_seats;
            return $data;
        }
    }
    
    public function events_by_type($type_id){
        return $this->dao->events_by_type($type_id);
    }
    
    public function event_count_by_type($type_id){
        return $this->dao->event_count_by_type($type_id);
    }
    
    public function get_upcoming_events_by_venue($venue_id){
        return $this->dao->get_upcoming_events_by_venue($venue_id);
    }
    
    public function upcoming_events_for_performer($performer_id){
        $results= $this->get_upcoming_events();
        
        $events= array();
        foreach($results as $event){
            if(in_array($performer_id, $event->performer)){
                $events[]= $event;
            }
        }
        return $events;
    }
    
    public function admin_event_hover_popup($event){
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $global_settings= $setting_service->load_model_from_db();
        $date_format = get_option('date_format');
        $time_format = get_option('time_format');
        $date_time_format = $date_format . ' ' . $time_format;
        $price= (abs(floatval($event->ticket_price))<=0) ? __('Free', 'eventprime-event-calendar-management') : em_currency_symbol().$event->ticket_price;
        $time_str = '';
        if (em_compare_event_dates($event->id)) {
            //$day = date('D, M d, Y',$event->start_date);
            $day = date_i18n($date_format,$event->start_date);
            $start_time = date_i18n($time_format,$event->start_date);
            $end_time = date_i18n($time_format,$event->end_date);

            $time_str .= $day . ' ' . $start_time . ' to ' . $end_time;
        } 
        else
        {
            $time_str .= date_i18n($date_time_format,$event->start_date) . ' - ' . date_i18n($date_time_format,$event->end_date);
        }
        $venue_address = em_get_term_meta($event->venue,'address',true);
        $address= !empty($venue_address) ? $venue_address : '';
        $performer_str='';
        $performer_service = EventM_Factory::get_service('EventM_Performer_Service');
        $type_service = EventM_Factory::get_service('EventTypeM_Service');
        if(!empty($event->enable_performer) && !empty($event->performer)) {
                $performers = array();
                foreach ($event->performer as $p_id) {
                    $performer = $performer_service->load_model_from_db($p_id);
                    $cover_image = $performer_service->get_image($performer->id);
                    if (!empty($cover_image)) {
                        array_push($performers, $cover_image);
                    }
                }
                if (!empty($performers)) {
                    $event_arr['performers'] = '<ul>';
                    foreach ($performers as $img_url) {
                        $performer_str .= "<li><img src='$img_url' /></li>";
                    }
                    $performer_str .= '</ul>';
                }
        }
        $booking_status='';
        $capacity = em_event_seating_capcity($event->id);
        if($event->enable_booking==1 && empty($event->hide_booking_status) && !empty($capacity)){
               $sum = $this->booked_seats($event->id);
               $width= ($sum / $capacity) * 100;
               $booking_status= array('stats'=>$sum.' / '.$capacity,'width'=>$width);
        }
        $popup_html = '<div class="em_event_detail_popup" style="display:none">
                            <div class="ep_event_detail_popup_head em_bg dbfl">
                                <div class="ep_event_title difl">'.$event->name.'</div>
                                <div class="ep_event_price difr">'.$price.'</div>
                            </div>
                            <div class="ep-event-detail-wrap">
                                <div class="ep-event-detail-row dbfl"><i class="material-icons">access_time</i><span class="ep_event_time">'.$time_str.'</span></div>';
        if(!empty($address)){
                $popup_html .= '<div class="ep-event-location ep-event-detail-row dbfl"><i class="material-icons">location_on</i><span class="ep_event_venue">'.$address.'</span></div>';
        }   
        if(!empty($performer_str)){
                $popup_html .= '<div class="ep-event-performers ep-event-detail-row dbfl"><span class="ep-event-performers-title">'.__('Performers','eventprime-event-calendar-management').'</span><div class="ep_event_performers">'.$performer_str.'</div></div>';
        }
        if(!empty($booking_status)){
                $popup_html .= '<div class="event_booking_details ep-event-detail-row">'.
                                    __("Booking Status", 'eventprime-event-calendar-management').
                                '<div class="status">'.$booking_status['stats'].'</div>
                                 <div class="dbfl">
                                    <div id="progressbar" class="em_progressbar dbfl">
                                        <div class="em_progressbar_fill em_bg" style="width:'.$booking_status['width'].'%"></div>
                                    </div>
                                </div></div>'; 
        }
        
        $popup_html .= '<div class="ep-shortcode dbfl"><i class="material-icons">code</i>[em_event id="'.$event->id.'"]</div>';
        $link= '<div class="ep-event-link ep-event-dash"><a href="'.admin_url('admin.php?page=em_dashboard&post_id='.$event->id).'">'.__('Dashboard','eventprime-event-calendar-management').'</a></div>';
        $delete_link= '<div class="ep-event-link ep-event-delete"><a href="#" data-id="'.$event->id.'"><i class="material-icons">delete_forever</i></a></div>';
        $settings_link= '<div class="ep-event-link ep-event-setting"><a style="float:right;" href="'.admin_url('admin.php?page=em_dashboard&tab=setting&post_id='.$event->id).'">'.__('Settings','eventprime-event-calendar-management').'</a></div>';
        $popup_html .= '<div class="ep-dashboard-link dbfl">'.$link.$delete_link.$settings_link.'</div>';
        $popup_html .= '</div></div>';
        return $popup_html;
    }
    
    public function get_events_by_venue($venue_id){
        return $this->dao->get_events_by_venue($venue_id);
    }
    
    public function get_events_by_user($user_id = 0){
        $events = array();
        $post_args = array(
            'author' => $user_id,
            'order' => 'DESC'
        );
        $event_posts = $this->dao->get_events($post_args);
        foreach($event_posts as $event_post) {
            $events[] = $this->load_model_from_db($event_post->ID);
        }
        return $events;
    }
    
    // Check if booking is enabled and seating capacity allows for new booking
    public function is_bookable($event){
        if(!empty($event->enable_booking)){
            // first check for recurring event and recurrence automatic booking
            if((isset($event->parent) && !empty($event->parent)) && (isset($event->enable_recurrence_automatic_booking) && !empty($event->enable_recurrence_automatic_booking))){
                return false;
            }
            // Now check for capacity
            if(!empty($event->seating_capacity)){
                $total_booked_seats= $this->booked_seats($event->id);
                if($total_booked_seats<$event->seating_capacity){
                    return true;
                }
            }
            else {
                return true;
            }
        }
        return false;
    }
    
    public function get_ical_file() {
        if (!is_admin()) {
            $event_id = absint(event_m_get_param('event'));
            if ($event_id == 0) {
                // Do nothing
            } else {
                $download_format = sanitize_text_field(event_m_get_param('download'));
                if ($download_format === 'ical') {
                    $event = $this->load_model_from_db($event_id);
                    $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
                    $options = $setting_service->load_model_from_db();
                    $event_url = add_query_arg(array('event'=>$event->id),get_permalink($options->events_page));

                    $ical  = "BEGIN:VCALENDAR"."\r\n";
                    $ical .= "VERSION:2.0"."\r\n";
                    $ical .= "METHOD:PUBLISH"."\r\n";
                    $ical .= "CALSCALE:GREGORIAN"."\r\n";
                    $ical .= "PRODID:-//WordPress - EPv".EVENTPRIME_VERSION."//EN"."\r\n";
                    //$ical .= "X-WR-CALNAME:WordPress"."\r\n";
                    $ical .= "X-ORIGINAL-URL:".home_url().'/'."\r\n";

                    $ical .= "BEGIN:VEVENT"."\r\n";
                    $ical .= "UID:EP-".md5(strval($event->id))."@".em_get_site_domain()."\r\n";
                    $ical .= "DTSTART:".gmdate('Ymd\\THi00', $event->start_date)."\r\n";
                    $ical .= "DTEND:".gmdate('Ymd\\THi00', $event->end_date)."\r\n";
                    $ical .= "DTSTAMP:".gmdate('Ymd\\THi00')."\r\n";
                    $ical .= "CREATED:".get_the_date('Ymd', $event->id)."\r\n";
                    $ical .= "LAST-MODIFIED:".get_the_modified_date('Ymd', $event->id)."\r\n";
                    $ical .= "SUMMARY:".html_entity_decode($event->name, ENT_NOQUOTES, 'UTF-8')."\r\n";
                    $ical .= "DESCRIPTION:".html_entity_decode(trim(strip_tags($event->description)), ENT_NOQUOTES, 'UTF-8')."\r\n";
                    $ical .= "URL:".$event_url."\r\n";

                    if ($event->venue != 0) {
                        $venue_serv = EventM_Factory::get_service('EventM_Venue_Service');
                        $venue = $venue_serv->load_model_from_db($event->venue);
                        $ical .= "LOCATION:".trim(strip_tags($venue->address))."\r\n";
                    }

                    $cover_image_id = get_post_thumbnail_id($event->id);
                    if (!empty($cover_image_id) && $cover_image_id > 0) {
                        $cover_image_url = $this->get_event_cover_image($event->id,'full');
                        $ical .= "ATTACH;FMTTYPE=".get_post_mime_type($cover_image_id).":".$cover_image_url."\r\n";
                    }

                    $ical .= "END:VEVENT"."\r\n";
                    $ical .= "END:VCALENDAR";

                    header('Content-type: application/force-download; charset=utf-8');
                    header('Content-Disposition: attachment; filename="ep-event-'.$event->id.'.ics"');

                    echo $ical;
                    exit;
                }
            }
        }
    }
    
    public function update_model($model){
        $this->dao->save($model);
    }
    
    public function set_meta($id,$key,$value){
         $this->dao->set_meta($id,$key,$value);
    }
    
    public function get_meta($id,$key){
        return $this->dao->get_meta($id,$key);
    }

    public function get_wishlist_events_by_user($user_id = 0){
        $events = array();
        $event_posts = get_user_meta($user_id, 'em_wishlist', true);
        if(!empty($event_posts)){
            foreach($event_posts as $event_post_key => $event_post_value) {
                $events[] = $this->load_model_from_db($event_post_key);
            }
        }
        return $events;
    }

    public function get_book_now_button_for_event($event_model){
        $book_now_btn = '<div>';
        if($this->is_bookable($event_model)){
            $current_ts = em_current_time_by_timezone();
            if($event_model->status=='expired'){
                $book_now_btn .= __("Bookings Expired","eventprime-event-calendar-management");
            } elseif($current_ts>$event_model->last_booking_date){
                $book_now_btn .= __("Bookings Closed","eventprime-event-calendar-management");
            } elseif($current_ts<$event_model->start_booking_date){
                $book_now_btn .= __("Bookings not started yet","eventprime-event-calendar-management");
            } else {
                $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
                $global_settings= $setting_service->load_model_from_db();

                if(is_user_logged_in()){
                    $currency_symbol= em_currency_symbol();
                    if ($event_model->ticket_price > 0){
                        $currencyHtml = ' - ' . $currency_symbol.$event_model->ticket_price;;
                    } else {
                        $currencyHtml = '';
                    }
                    $book_now_btn .= '<form action="'.get_permalink($global_settings->booking_page).'" method="post" name="em_booking">
                                            <button class="em-wishlist-book-now-btn " name="tickets" onclick="em_event_booking('.$event_model->id.')" class="em_header_button" id="em_booking">
                                                <i class="fa fa-ticket" aria-hidden="true"></i>'
                                                .__("Book Now","eventprime-event-calendar-management").$currencyHtml.'
                                            </button>
                                            <input type="hidden" name="event_id" value="'.$event_model->id.'" />
                                            <input type="hidden" name="venue_id" value="'.$event_model->venue.'" />
                                        </form>';
                } else {
                    $book_now_btn .= '<button class="em_profile_wishlist_btn em_header_button kf-tickets" target="_blank" href="'.add_query_arg("event_id",$event_model->id,get_permalink($global_settings->profile_page)).'">'._e("Book Now","eventprime-event-calendar-management").'</button>';
                }
            }
        } else {
            if((isset($event_model->parent) && !empty($event_model->parent)) && (isset($event_model->enable_recurrence_automatic_booking) && !empty($event_model->enable_recurrence_automatic_booking))){
                $book_now_btn .= __("Bookings not allowed","eventprime-event-calendar-management");
            }
            else{
                $book_now_btn .= __("Bookings closed","eventprime-event-calendar-management");
            }
        }
        $book_now_btn .= '</div>';
        return $book_now_btn;
    }
}