<?php

if (!defined( 'ABSPATH')){
    exit;
}

class EventM_Notification_Service {
    
    public static function booking_confirmed($booking)
    {  
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        
        if(empty($gs->send_booking_confirm_email))
           return;
        
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        if(empty($event->id))
            return;
        
        // id with date and time
        $bookingIdDate = $booking->id . ' - ' .em_showDateTime(em_get_post_meta($event->id,'start_date',true),true);
        self::configure_mail();
        $user= get_user_by('ID',$booking->user);
        $mail_body =  $gs->booking_confirmed_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$bookingIdDate, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $sub_total = $booking->order_info['quantity'] * $booking->order_info['item_price'];
        $total_discount = 0;
        if(isset($booking->order_info['discount']) && !empty($booking->order_info['discount'])){
            $sub_total -= $booking->order_info['discount'];
            $total_discount += $booking->order_info['discount'];
        }
        if(isset($booking->order_info['coupon_discount']) && !empty($booking->order_info['coupon_discount'])){
            $sub_total -= $booking->order_info['coupon_discount'];
            $total_discount += $booking->order_info['coupon_discount'];
        }
        if(isset($booking->order_info['ebd_discount_amount']) && !empty($booking->order_info['ebd_discount_amount'])){
            $sub_total -= $booking->order_info['ebd_discount_amount'];
            $total_discount += $booking->order_info['ebd_discount_amount'];
        }
        $mail_body = str_replace("$(Subtotal)", $sub_total, $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $total_discount, $mail_body);
        $mail_body = str_replace("(User Email)", $user->email, $mail_body);
       
        /* send Mail to User */    
        $to = $user->user_email;
        $subject = __('Booking Confirmation','eventprime-event-calendar-management');
        if(empty($gs->disable_frontend_email)){
            wp_mail( $to, $subject, $mail_body);
        }

        /* send Mail to Admin */   
        $mail_body = file_get_contents(EM_BASE_URL.'includes/mail/admin_confirm.html');
        $to = get_option('admin_email'); 
        $subject = __('Booking Confirmation','eventprime-event-calendar-management');
        $mail_body = str_replace("#ID", $booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("(User Email)", $user->user_email, $mail_body);
        $mail_body = str_replace("Booking Date", em_showDateTime(em_get_post_meta($event->id,'start_date',true),true), $mail_body);
        if(empty($gs->disable_admin_email)){
            wp_mail( $to, $subject, $mail_body);
        }
    }
    
    public static function booking_pending($booking)
    {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs = $setting_service->load_model_from_db();
        if(empty($gs->send_booking_pending_email))
           return;
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
        
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;
        
        $mail_body =  $gs->booking_pending_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("$(Subtotal)", ($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount']-$booking->order_info['coupon_discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount']+$booking->order_info['coupon_discount'], $mail_body);
        $to = $user->user_email;
        $subject = __('Booking Pending','eventprime-event-calendar-management');
        if(empty($gs->disable_frontend_email)){
            wp_mail( $to, $subject, $mail_body );
        }
        
        // Admin email
        $to = get_option('admin_email');
        $subject = __('Booking Pending','eventprime-event-calendar-management');        
        $body = sprintf(__('User %s has Booking Pending with Booking ID #%d.','eventprime-event-calendar-management'),$user->user_email,$booking->id);
        if(empty($gs->disable_admin_email)){
            wp_mail( $to, $subject, $body);
        }
    }

    
    public static function booking_cancel($booking)
    {   
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        if(empty($gs->send_booking_cancellation_email))
           return; 
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return;
       
        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;            
            
        $mail_body =  $gs->booking_cancelation_email;
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("$(Subtotal)", ($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount']-$booking->order_info['coupon_discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount']+$booking->order_info['coupon_discount'], $mail_body);
        $to = $user->user_email;
        $subject = __('Booking Cancellation','eventprime-event-calendar-management');
        if(empty($gs->disable_frontend_email)){
            wp_mail( $to, $subject, $mail_body ); 
        }
        
        // Admin email
        $mail_body = file_get_contents(EM_BASE_URL.'includes/mail/admin_cancellation.html');  
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        
        $admin_email = get_option('admin_email'); 
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("(User Email)",$user->user_email, $mail_body);
        $to = $admin_email;
        $subject = __('Booking Cancellation','eventprime-event-calendar-management');
        if(empty($gs->disable_admin_email)){
            wp_mail( $to, $subject, $mail_body); 
        }
    }
    
    
    public static function reset_password_mail($booking,$new_user_password)
    {   
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
                
        if(empty($booking->id))
            return;
        
        self::configure_mail();
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;

        $mail_body= $gs->reset_password_mail;
        $mail_body = str_replace("@username",$user->user_email,$mail_body);
        $mail_body = str_replace("@password",$new_user_password,$mail_body);   
         
        
        $to = $user->user_email;
        $subject = __('New Password','eventprime-event-calendar-management');
        $body = $mail_body;
        wp_mail( $to, $subject, $body );
        
        $admin_email = get_option('admin_email'); 
        $to = $admin_email;
        $subject = __('Reset User Password','eventprime-event-calendar-management');        
        $body = sprintf(__('Password of user %s is Reset.','eventprime-event-calendar-management'),$user->user_email);
        wp_mail( $to, $subject, $body);
    }
    
    public static function booking_refund($booking)
    {
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        if(empty($gs->send_booking_refund_email))
           return;
        self::configure_mail();
        $booking_service = EventM_Factory::get_service('EventM_Booking_Service');
        if(!$booking instanceof EventM_Booking_Model){
            $booking= $booking_service->load_model_from_db($booking);
        }
        if(empty($booking->id))
            return false;
        $user= get_user_by('ID', $booking->user);
        if(empty($user))
            return false;

        $event_service = EventM_Factory::get_service('EventM_Service');
        $event= $event_service->load_model_from_db($booking->event);
        
        $venue_service= EventM_Factory::get_service('EventM_Venue_Service');
        $venue= $venue_service->load_model_from_db($event->venue);
        if(empty($venue->id)){
            $venue= false;
        }
        
        
        $mail_body =  $gs->booking_refund_email; 
        $mail_body= isset($booking->order_info['seat_sequences']) ? str_replace("(Seat No.)",implode(',',$booking->order_info['seat_sequences']), $mail_body) : str_replace("(Seat No.)", "Standing Event", $mail_body);
        $mail_body = str_replace("#ID",$booking->id, $mail_body);
        $mail_body = str_replace("Event Name", $event->name, $mail_body);
        $mail_body = str_replace("Event Venue",empty($venue->id) ? '' : $venue->address, $mail_body);
        $mail_body = str_replace("Venue Name",empty($venue->id) ? '' : $venue->name, $mail_body);
        $mail_body = str_replace("$(Subtotal)",($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount']-$booking->order_info['coupon_discount'], $mail_body);
        $mail_body = str_replace("(Quantity)", $booking->order_info['quantity'], $mail_body);
        $mail_body = str_replace("$(Price)", $booking->order_info['item_price'], $mail_body);
        $mail_body = str_replace("$(Discount)", $booking->order_info['discount']+$booking->order_info['coupon_discount'], $mail_body);
        
        $to = $user->user_email;
        $subject = __('Booking Refund','eventprime-event-calendar-management');
        if(empty($gs->disable_frontend_email)){
            wp_mail($to, $subject, $mail_body );
        }
        
        //Admin notification
        $admin_email = get_option('admin_email');  
        $to = $admin_email; 
        $subject = sprintf(__('Booking Refund on Booking ID# %d','eventprime-event-calendar-management'),$booking->id);        
        $body = sprintf(__('A refund of %s%0.2f has been issued to booking #%d for %s','eventprime-event-calendar-management'),$booking->order_info['currency'],($booking->order_info['quantity']*$booking->order_info['item_price'])-$booking->order_info['discount']-$booking->order_info['coupon_discount'],$booking->id,$event->name);
        if(empty($gs->disable_admin_email)){
            wp_mail($to, $subject, $body);
        }
    }
    
    public static function user_registration($user_data=null ) { 
        self::configure_mail();
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $gs= $setting_service->load_model_from_db();
        $mail_body= $gs->registration_email_content;
        $mail_body = str_replace("@username",$user_data->email,$mail_body);
        $mail_body = str_replace("@password",$user_data->password,$mail_body);
        
        //$body_content .= "Your auto generated password is ".$user_data->password;
        $registration_email_subject= $gs->registration_email_subject;
        
        if(empty($registration_email_subject)){
            $registration_email_subject= get_bloginfo('name');
        }
        
        if(!empty($user_data)){
            wp_mail( $user_data->email, $registration_email_subject, $mail_body );
            $admin_email = get_option('admin_email'); 
            $to = $admin_email;
            $subject = __('New User Registered','eventprime-event-calendar-management'); 
            $body = sprintf(__('New user %s has Registered','eventprime-event-calendar-management'),$user_data->email);
            wp_mail( $to, $subject, $body);
            return true;
        }
        return false;
    }
    
    public static function event_submitted($event_id)
    {  
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $event_service = EventM_Factory::get_service('EventM_Service');
        $gs = $setting_service->load_model_from_db();
        if(empty($gs->send_event_submitted_email))
           return;

        if(empty($gs->event_submitted_email) || is_null($gs->event_submitted_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_submitted.html');
            $gs->event_submitted_email = ob_get_clean();
        }
        $userEmail = wp_get_current_user()->user_email;
        if(empty($userEmail)){
            $userEmail = 'User';
        }
        self::configure_mail();
        $mail_body = $gs->event_submitted_email;
        $mail_body = str_replace("@UserEmail",$userEmail,$mail_body);
        $mail_body = str_replace("@EventName",get_the_title($event_id),$mail_body);
        $mail_body = str_replace("@EventStartDate",em_showDateTime(em_get_post_meta($event_id,'start_date',true),true),$mail_body);
        $mail_body = str_replace("@EventEndDate",em_showDateTime(em_get_post_meta($event_id,'end_date',true),true),$mail_body);
       
        /* Send Mail to Admin */    
        $to = get_option('admin_email');
        $subject = __('New Event Submitted','eventprime-event-calendar-management');
        if(empty($gs->disable_admin_email)){
            wp_mail($to, $subject, $mail_body);
        }
    }
    
    public static function event_approved($event_id)
    {  
        $setting_service = EventM_Factory::get_service('EventM_Setting_Service');
        $event_service = EventM_Factory::get_service('EventM_Service');
        $gs = $setting_service->load_model_from_db();
        if(empty($gs->send_event_approved_email))
           return;
        $event = $event_service->load_model_from_db($event_id);
        
        if(empty($gs->event_approved_email) || is_null($gs->event_approved_email)) {
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_approved.html');
            $gs->event_submitted_email = ob_get_clean();
        }
        
        self::configure_mail();
        $mail_body = $gs->event_approved_email;
        $mail_body = str_replace("@UserName",get_the_author_meta('display_name',$event->user),$mail_body);
        $mail_body = str_replace("@EventName",$event->name,$mail_body);
        $mail_body = str_replace("@SiteURL",site_url(),$mail_body);
        $mail_body = str_replace("@EventLink",add_query_arg(array('event'=>$event->id),get_permalink($gs->events_page)),$mail_body);

        /* Send Mail to Event Author */
        $to = get_the_author_meta('user_email',$event->user);
        $subject = __('Event Published','eventprime-event-calendar-management');
        if(empty($gs->disable_frontend_email)){
            wp_mail($to, $subject, $mail_body);
        }
    }
    
    private static function configure_mail()
    {  
        add_filter('wp_mail_content_type', 'em_set_mail_content_type_html');
        add_filter('wp_mail_from', 'em_set_mail_from');
        add_filter('wp_mail_from_name', 'em_set_mail_from_name');
    }  
}