<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class EventM_Setting_Service {
    
    private $dao;
    private static $instance = null; 
    
    private function __construct() {
        $this->dao= new EventM_Global_Settings_DAO();
    }
    
    public static function get_instance()
    {   
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    /*
     * Loading settings data on REST call
     */
    public function load_edit_page()
    {
        global $wp_roles;
        $options= $this->load_model_from_db();
        $options= apply_filters('em_load_gs_ext_options',$options);
        $options->pages= $this->load_global_pages();
        $options->base_path= EM_BASE_URL.'includes/admin/template/';
        $options->currencies= em_array_to_options(EventM_Constants::$currencies);
        $options->time_formats= $this->load_time_formats();
        $options->status_list= array(
            "publish" => __('Active','eventprime-event-calendar-management'),
            "draft" => __('Draft','eventprime-event-calendar-management')
        );
        $options->default_calendar_date = date(get_option('date_format'),$options->default_calendar_date);
        $options->user_roles = array();
        $userRoles = $wp_roles->roles;
        if(!empty($userRoles)){
            foreach ($userRoles as $key => $value) {
                $options->user_roles[$key] = $value['name'];
            }
        }
        return $options;
    }

    public function save($model)
    {   
       $model= $this->format_model_to_save($model);
       $this->dao->save($model);
    }
    
    
    public function load_global_pages(){
        $list= array();
        $pages = get_pages(array(
            'post_status'=>'publish',
             'numberposts'=>-1,
            ) );
         
         if (!empty($pages)) {
           foreach ($pages as $page) {
                $tmp = new stdClass();
                $tmp->id = $page->ID;
                $tmp->name = $page->post_title;
                $list[] = $tmp;
            }
       }
       return $list; 
    }
    
    public function load_time_formats() {
        $list = array(
            'h:mmt' => __('12-hour','eventprime-event-calendar-management'),
            'HH:mm' => __('24-hour','eventprime-event-calendar-management')
        );
        
        return $list;
    }
    
    public function load_model_from_db()
    {
        $model= $this->dao->get();
        $model->gcal_sharing= absint($model->gcal_sharing);
        $model->events_page= absint($model->events_page);
        $model->event_types= absint($model->event_types);
        $model->performers_page= absint($model->performers_page);
        $model->profile_page= absint($model->profile_page);
        $model->venues_page= absint($model->venues_page);
        $model->booking_page= absint($model->booking_page);
        $model->event_submit_form= absint($model->event_submit_form);
        $model->paypal_processor= absint($model->paypal_processor);
        $model->payment_test_mode= absint($model->payment_test_mode);
        if(empty($model->event_submitted_email) || is_null($model->event_submitted_email)){
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_submitted.html');
            $model->event_submitted_email = ob_get_clean();
        }
        if(empty($model->event_approved_email) || is_null($model->event_approved_email)){
            ob_start();
            include(EM_BASE_DIR . 'includes/mail/event_approved.html');
            $model->event_approved_email = ob_get_clean();
        }
        if(empty($model->ues_confirm_message)){
            $model->ues_confirm_message = __('Thank you for submitting your event. We will review and publish it soon.','eventprime-event-calendar-management');
        }
        if(empty($model->ues_login_message)){
            $model->ues_login_message = __('Please login to submit your event.','eventprime-event-calendar-management');
        }
        if(empty($model->default_calendar_date)){
            $model->default_calendar_date = em_get_local_timestamp();
        }
        if(empty($model->ues_restricted_submission_message)){
            $model->ues_restricted_submission_message = __('You are not authorised to access this page. Please contact with your administrator.','eventprime-event-calendar-management');
        }
        return $model;
    }
    
    private function format_model_to_save($model){
        $model->gcal_sharing= absint($model->gcal_sharing);
        $model->events_page= absint($model->events_page);
        $model->event_types= absint($model->event_types);
        $model->performers_page= absint($model->performers_page);
        $model->profile_page= absint($model->profile_page);
        $model->venues_page= absint($model->venues_page);
        $model->booking_page= absint($model->booking_page);
        $model->event_submit_form= absint($model->event_submit_form);
        $model->paypal_processor= empty($model->paypal_processor) ? 0 : 1;
        $model->payment_test_mode= empty($model->payment_test_mode) ? 0 : 1;
        $model->paypal_email=  sanitize_email($model->paypal_email);
        $model->paypal_api_username= sanitize_text_field($model->paypal_api_username);
        $model->paypal_api_password= sanitize_text_field($model->paypal_api_password);
        $model->paypal_api_sig= sanitize_text_field($model->paypal_api_sig);
        $model->ues_confirm_message= sanitize_text_field($model->ues_confirm_message);
        $model->ues_login_message= sanitize_text_field($model->ues_login_message);
        $model->ues_default_status= sanitize_text_field($model->ues_default_status);
        $model->enable_default_calendar_date= absint($model->enable_default_calendar_date);
        $model->default_calendar_date= em_timestamp(sanitize_text_field($model->default_calendar_date),get_option('date_format'));
        $model->ues_restricted_submission_message= sanitize_text_field($model->ues_restricted_submission_message);
        return $model;
    }
    
    public function map_request_to_model($model=null)
    {  
        $settings= new EventM_Global_Settings_Model();
        $data= (array) $model;
        
        if(!empty($data) && is_array($data))
        {
            foreach($data as $key=>$val)
            {
                if(property_exists($settings, $key)){
                    $settings->{$key}= $val;
                }
            }
        }
        return $settings;
    }
        
}