eventMagicApp.controller('emEventSubmitCtrl', function($scope, $http, EMRequest) {
    $scope.data = {};
    $scope.formErrors = new Array();
    $scope.submitted = false;
    $scope.requestInProgress = false;
    $ = jQuery;
   
    $scope.progressStart = function() {
        $scope.requestInProgress = true;
    }
    
    $scope.progressStop = function() {
        $scope.requestInProgress = false;
    }
    
    // Called at initialization
    $scope.initialize = function() {
        $scope.progressStart();

        // HTTP request to load data
        EMRequest.send('em_frontend_event_submit_form_strings',$scope.data).then(function(response){
            var responseBody = response.data;
            if(responseBody.success){
                $scope.data = responseBody.data;
                $scope.data.settings.venue = $scope.data.settings.venues[0].id;
                $scope.data.settings.event_type = $scope.data.settings.event_types[0].id;
                $scope.initializeDateTimePickers();
            }
            $scope.progressStop();
        });
    }
    
    $scope.initializeDateTimePickers = function() {
        var minDate = new Date();
        var maxDate;
        $("#event_start_date").datetimepicker({controlType: 'select',oneLine: true,changeYear: true,timeFormat: 'HH:mm',dateFormat: "mm/dd/yy", timeInput: true});
        $("#event_end_date").datetimepicker({controlType: 'select',oneLine: true,changeYear: true,timeFormat: 'HH:mm',dateFormat: "mm/dd/yy", timeInput: true});
        $("#event_start_booking_date").datetimepicker({controlType: 'select',oneLine: true,timeFormat: 'HH:mm',changeYear: true,dateFormat: "mm/dd/yy", timeInput: true});
        $("#event_last_booking_date").datetimepicker({controlType: 'select',oneLine: true,timeFormat: 'HH:mm',changeYear: true,dateFormat: "mm/dd/yy", timeInput: true});
    }
    
    $scope.submitEvent = function(isValid){
        if(isValid){
            if($('#event_description').is(':visible')){
                $scope.data.settings.description = $('#event_description').val();
            } else {
                if(typeof tinymce!="undefined"){
                    $scope.data.settings.description= tinymce.get('event_description').getContent();
                }
            }
            $newVenueAddressInp = $("#em-pac-input");
            if($newVenueAddressInp.length > 0){
                $scope.data.settings.new_venue_address = $newVenueAddressInp.val();
            }
            $scope.progressStart();
            EMRequest.send('em_submit_frontend_event',$scope.data.settings).then(function(response){
                $scope.progressStop();
                var responseBody= response.data;
                if(responseBody.success){
                    EMRequest.send('em_event_submitted_mail',responseBody.data).then(function(response){});
                    $scope.submitted = true;
                } else {
                    if(responseBody.data.hasOwnProperty('errors')){
                        $scope.formErrors= responseBody.data.errors;
                    }
                }
                jQuery('html, body').animate({ scrollTop: 0 }, 'slow');
            });
        }        
    }

    $scope.getFrontCapacity = function(key){
        if($scope.data.settings.venue == 'new_venue'){
            if(key != ''){
                $scope.setupMap();
            }
        }
    }

    // setup map on add new event site
    $scope.setupMap = function () {
        var gmarkers = []; // To store all the markers
        // Initializing Map
        var map = new google.maps.Map(document.getElementById('map'), {
            zoom: 8,
            center: {lat: 40.731, lng: -73.997}
        });
        var latEl = $("#em_venue_lat");
        var lngEl = $("#em_venue_long");
        var addressEl=$("#em-pac-input");
        input = document.getElementById('em-pac-input'); //Searchbox
        var types = document.getElementById('type-selector');
        map.controls[google.maps.ControlPosition.TOP_LEFT].push(input);
        map.controls[google.maps.ControlPosition.TOP_LEFT].push(types);
        var geocoder = new google.maps.Geocoder;
        var infowindow = new google.maps.InfoWindow;
        var autocomplete = new google.maps.places.SearchBox(input);
        autocomplete.bindTo('bounds', map);
        var marker = new google.maps.Marker({
            map: map,
            anchorPoint: new google.maps.Point(0, -29)
        });
        
        resetMarkers=function(){
            for (i = 0; i < gmarkers.length; i++) {
                gmarkers[i].setMap(null);
            }
        }      
        // Listener for searchbox changes.
        autocomplete.addListener('places_changed', function () {
            resetMarkers();
            var places = autocomplete.getPlaces();
            if(places.length==0)
                return;
            var place=places[0];
            if (!place.geometry) {
                return;
            }
            // If the place has a geometry, then present it on a map.
            if (place.geometry.viewport) {
                map.fitBounds(place.geometry.viewport);
            } else {
                map.setCenter(place.geometry.location);
                map.setZoom(8);  // Why 17? Because it looks good.
            }
            var marker = new google.maps.Marker({
                position: place.geometry.location,
                map: map
            });
            var address = '';
            if (place.address_components) {
                address = [
                    (place.address_components[0] && place.address_components[0].short_name || ''),
                    (place.address_components[1] && place.address_components[1].short_name || ''),
                    (place.address_components[2] && place.address_components[2].short_name || '')
                ].join(' ');
            }
            infowindow.setContent('<div><strong>' + place.name + '</strong><br>' + address);
            infowindow.open(map,marker);
            gmarkers.push(marker);
        });
    
        map.addListener('click', function (e) {
            resetMarkers();
            latLng = e.latLng;
            var latlng = {lat: parseFloat(latLng.lat()), lng: parseFloat(latLng.lng())};
            geocoder.geocode({'location': latlng}, function (results, status) {
                if (status === google.maps.GeocoderStatus.OK) {
                    if (results[1]) {
                        map.setZoom(13);
                        var marker = new google.maps.Marker({
                            position: latlng,
                            map: map
                        });
                        input.value = results[1].formatted_address;
                        infowindow.setContent(results[1].formatted_address);
                        infowindow.open(map, marker);
                        gmarkers.push(marker);
                    }
                }
            });
        });
    }
})
.directive("fileInput", ['$http', '$parse', function($http, $parse){  
    return{  
        link: function($scope, element, attrs){  
            element.on("change", function(event){  
                var files = event.target.files;
                var fileType = files[0].type;
                var allowedFileType = ['image/jpeg', 'image/jpg', 'image/png', 'image/gif'];
                if(allowedFileType.indexOf(fileType) > -1){
                    $scope.progressStart();
                    var ajUrl = em_ajax_object.ajax_url + "?action=em_upload_image_from_frontend";
                    $scope.form = [];
                    $scope.form.image = element[0].files[0];
                    $http({
                        method  : 'POST',
                        url     : ajUrl,
                        processData: false,
                        transformRequest: function (data) {
                            var formData = new FormData();
                            formData.append("image", $scope.form.image);  
                            return formData;  
                        },  
                        data : $scope.form,
                        headers: {
                            'Content-Type': undefined
                        }
                    }).success(function(response){
                        $scope.progressStop();
                        $scope.data.settings.attachment_id = response.data.attachment_id;
                        var reader = new FileReader();
                        reader.onload = function (event) {
                            $scope.PreviewImage = event.target.result;
                            $scope.$apply();
                        };
                        reader.readAsDataURL(event.target.files[0]);
                    });
                }
                else{
                    alert("Only Image File Allowed");
                    $scope.PreviewImage = null;
                    angular.element("input[type='file']").val(null);
                }
            });  
        }  
    }  
}]);